@@ -232,6 +232,15 @@ ui_check_options_and_arguments(struct arithmeticparams *p)
   size_t numfits=0, numhdus=0;
   gal_list_str_t *token, *hdu;
 
+  /* First, make sure that any tokens are actually given. */
+  if(p->tokens==NULL)
+    error(EXIT_FAILURE, 0, "no input tokens. Please specify a filename or "
+          "number (as operands) along with operator(s) as input. Please run "
+          "any of the following commands for more information.\n\n"
+          "    $ astarithmetic --help           # Short info.\n"
+          "    $ info astarithmetic             # Full invocation "
+          "documentation.\n");
+
   /* The input tokens are put in a lastin-firstout (simple) linked list, so
      change them to the correct order so the order we pop a token is the
      same order that the user input a value. Note that for the options this
@@ -256,8 +265,8 @@ ui_check_options_and_arguments(struct arithmeticparams *p)
           if(output_checked==0)
             {
               if(p->cp.output)
-                gal_checkset_check_remove_file(p->cp.output, 0,
-                                               p->cp.dontdelete);
+                gal_checkset_writable_remove(p->cp.output, 0,
+                                             p->cp.dontdelete);
               else
                 p->cp.output=gal_checkset_automatic_output(&p->cp, token->v,
                                                            "_arith.fits");
@@ -272,14 +281,21 @@ ui_check_options_and_arguments(struct arithmeticparams *p)
         token->v[0]='-';
     }
 
-  /* Count the number of HDU values and check if its not less than the
-     number of input FITS images. */
-  for(hdu=p->hdus; hdu!=NULL; hdu=hdu->next) ++numhdus;
-  if(numhdus<numfits)
-    error(EXIT_FAILURE, 0, "not enough HDUs. There are %zu input FITS "
-          "files, but only %zu HDUs. You can use the `--hdu' (`-h') option "
-          "to specify the number or name of a HDU for each FITS file",
-          numfits, numhdus);
+  /* Count the number of HDU values (if globalhdu isn't given) and check if
+     its not less than the number of input FITS images. */
+  if(p->globalhdu)
+    { if(p->hdus) { gal_list_str_free(p->hdus, 1); p->hdus=NULL; }; }
+  else
+    {
+      for(hdu=p->hdus; hdu!=NULL; hdu=hdu->next) ++numhdus;
+      if(numhdus<numfits)
+        error(EXIT_FAILURE, 0, "not enough HDUs. There are %zu input FITS "
+              "files, so the `--hdu' (`-h') option must be called %zu "
+              "times (once for each FITS file). If the HDU value is the "
+              "same for all the files, you may use `--globalhdu' (`-g') to "
+              "specify a single HDU to be used for any number of input "
+              "files", numfits, numfits);
+    }
 }
 
 
@@ -382,7 +398,9 @@ ui_read_check_inputs_setup(int argc, char *argv[], struct arithmeticparams *p)
 void
 freeandreport(struct arithmeticparams *p, struct timeval *t1)
 {
+  /* Free the simple strings. */
   free(p->cp.output);
+  if(p->globalhdu) free(p->globalhdu);
 
   /* If there are any remaining HDUs in the hdus linked list, then
      free them. */
