@@ -73,6 +73,37 @@ struct argp_option program_options[] =
       GAL_OPTIONS_NOT_SET
     },
 
+    {
+      "la",
+      UI_KEY_LA,
+      "STR",
+      0,
+      "Libtool `.la' to use instead of default.",
+      GAL_OPTIONS_GROUP_INPUT,
+      &p->la,
+      GAL_TYPE_STRING,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
+
+    {
+      "tag",
+      UI_KEY_TAG,
+      "STR",
+      0,
+      "Libtool `--tag': programming language.",
+      GAL_OPTIONS_GROUP_INPUT,
+      &p->tag,
+      GAL_TYPE_STRING,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
+
+
+
+
 
     {
       "debug",
@@ -126,6 +157,19 @@ struct argp_option program_options[] =
       GAL_OPTIONS_NOT_MANDATORY,
       GAL_OPTIONS_NOT_SET
     },
+    {
+      "deletecompiled",
+      UI_KEY_DETELECOMPILED,
+      0,
+      0,
+      "Delete compiled program after running.",
+      GAL_OPTIONS_GROUP_OUTPUT,
+      &p->deletecompiled,
+      GAL_OPTIONS_NO_ARG_TYPE,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
 
     {0}
   };
