@@ -91,7 +91,7 @@ struct argp_option program_options[] =
     {
       0, 0, 0, 0,
       "Flux:",
-      ARGS_GROUP_FLUX
+      UI_GROUP_FLUX
     },
     {
       "fluxlow",
@@ -99,7 +99,7 @@ struct argp_option program_options[] =
       "FLT",
       0,
       "Lower flux truncation value.",
-      ARGS_GROUP_FLUX,
+      UI_GROUP_FLUX,
       &p->fluxlowstr,
       GAL_TYPE_STRING,
       GAL_OPTIONS_RANGE_ANY,
@@ -112,7 +112,7 @@ struct argp_option program_options[] =
       "FLT",
       0,
       "Higher flux truncation value.",
-      ARGS_GROUP_FLUX,
+      UI_GROUP_FLUX,
       &p->fluxhighstr,
       GAL_TYPE_STRING,
       GAL_OPTIONS_RANGE_ANY,
@@ -125,7 +125,7 @@ struct argp_option program_options[] =
       "INT",
       0,
       "Maximum byte value for all color channels.",
-      ARGS_GROUP_FLUX,
+      UI_GROUP_FLUX,
       &p->maxbyte,
       GAL_TYPE_UINT8,
       GAL_OPTIONS_RANGE_GE_0,
@@ -138,7 +138,7 @@ struct argp_option program_options[] =
       0,
       0,
       "Set value of fluxlow as the minimum byte value.",
-      ARGS_GROUP_FLUX,
+      UI_GROUP_FLUX,
       &p->flminbyte,
       GAL_OPTIONS_NO_ARG_TYPE,
       GAL_OPTIONS_RANGE_0_OR_1,
@@ -151,7 +151,7 @@ struct argp_option program_options[] =
       0,
       0,
       "Set value of fluxhigh as the maximum byte value.",
-      ARGS_GROUP_FLUX,
+      UI_GROUP_FLUX,
       &p->fhmaxbyte,
       GAL_OPTIONS_NO_ARG_TYPE,
       GAL_OPTIONS_RANGE_0_OR_1,
@@ -164,7 +164,7 @@ struct argp_option program_options[] =
       "STR",
       0,
       "Change pixel values `from_1:to_1,from_2:to_2`.",
-      ARGS_GROUP_FLUX,
+      UI_GROUP_FLUX,
       &p->changestr,
       GAL_TYPE_STRING,
       GAL_OPTIONS_RANGE_ANY,
@@ -177,7 +177,7 @@ struct argp_option program_options[] =
       0,
       0,
       "First truncate then change pixel values.",
-      ARGS_GROUP_FLUX,
+      UI_GROUP_FLUX,
       &p->changeaftertrunc,
       GAL_OPTIONS_NO_ARG_TYPE,
       GAL_OPTIONS_RANGE_0_OR_1,
@@ -190,7 +190,7 @@ struct argp_option program_options[] =
       0,
       0,
       "Invert the values in JPEG and EPS/PDF.",
-      ARGS_GROUP_FLUX,
+      UI_GROUP_FLUX,
       &p->invert,
       GAL_OPTIONS_NO_ARG_TYPE,
       GAL_OPTIONS_RANGE_0_OR_1,
