@@ -80,16 +80,6 @@ doc[] = GAL_STRINGS_TOP_HELP_INFO PROGRAM_NAME" will convert any of the "
 
 
 
-/* Option groups particular to this program */
-enum program_args_groups
-{
-  ARGS_GROUP_FLUX = GAL_OPTIONS_GROUP_AFTER_COMMON,
-};
-
-
-
-
-
 
 
 
@@ -433,8 +423,7 @@ ui_make_channels_ll(struct converttparams *p)
                   "for each input FITS image (in the same order)");
 
           /* Read in the array and its WCS information. */
-          data=gal_fits_img_read(name->v, hdu, p->cp.minmapsize);
-          data->wcs=gal_wcs_read(name->v, hdu, 0, 0, &data->nwcs);
+          data=gal_fits_img_read(name->v, hdu, p->cp.minmapsize, 0, 0);
           gal_list_data_add(&p->chll, data);
 
           /* A FITS file only has one channel. */
