@@ -40,10 +40,10 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 /* Macros */
 #define LOGFILENAME             PROGRAM_EXEC".log"
 #define FILENAME_BUFFER_IN_VERB 30
+#define MAXDIM                  2
 
 
-
-/* Modes of operation. */
+/* Modes to interpret coordinates. */
 enum crop_modes
 {
   IMGCROP_MODE_INVALID,         /* For sanity checks.     */
@@ -56,8 +56,7 @@ enum crop_modes
 
 
 /* The sides of the image keep the celestial coordinates of the four
-   sides of this image. With respect to the pixels they are.
-*/
+   sides of this image. With respect to the pixels they are. */
 struct inputimgs
 {
   char             *name;  /* File name of input image.                   */
@@ -81,43 +80,34 @@ struct cropparams
 {
   /* Directly from command-line */
   struct gal_options_common_params cp;  /* Common parameters.             */
-  struct gal_list_str_t       *inputs;  /* All input FITS files.          */
+  gal_list_str_t       *inputs;  /* All input FITS files.                 */
   size_t             hstartwcs;  /* Header keyword No. to start read WCS. */
   size_t               hendwcs;  /* Header keyword No. to end read WCS.   */
+  int                     mode;  /* Image or WCS mode.                    */
   uint8_t       zeroisnotblank;  /* ==1: In float or double, keep 0.0.    */
   uint8_t              noblank;  /* ==1: no blank (out of image) pixels.  */
   char                 *suffix;  /* Ending of output file name.           */
   size_t           checkcenter;  /* width of a box to check for zeros     */
-  size_t              iwidthin;  /* Image mode width (in pixels).         */
-  double                wwidth;  /* WCS mode width (in arcseconds).       */
-  double                    ra;  /* RA of one crop box center.            */
-  double                   dec;  /* Dec of one crop box center.           */
-  double                    xc;  /* Center point, one crop (FITS stnrd).  */
-  double                    yc;  /* Center point, one crop (FITS stnrd).  */
+  gal_data_t           *center;  /* Center position of crop.              */
+  gal_data_t            *width;  /* Width of crop when defined by center. */
   char                *catname;  /* Name of input catalog.                */
   char                 *cathdu;  /* HDU of catalog if its a FITS file.    */
   char                *namecol;  /* Filename (without suffix) of crop col.*/
-  char                  *racol;  /* Catalog RA column                     */
-  char                 *deccol;  /* Catalog Dec column                    */
-  char                   *xcol;  /* Catalog X column                      */
-  char                   *ycol;  /* Catalog Y column                      */
+  gal_list_str_t     *coordcol;  /* Column in catalog with coordinates.   */
   char                *section;  /* Section string.                       */
   char                *polygon;  /* Input string of polygon vertices.     */
   uint8_t           outpolygon;  /* ==1: Keep the inner polygon region.   */
-  char                *modestr;  /* ==1: will use X and Y coordiates.     */
 
   /* Internal */
-  int                     mode;  /* Image or WCS mode.                    */
   size_t                 numin;  /* Number of input images.               */
   size_t                numout;  /* Number of output images.              */
-  double                   *c1;  /* First coordinate from catalog.        */
-  double                   *c2;  /* Second coordinate from catalog.       */
+  double        **centercoords;  /* The center coordinates.               */
   char                  **name;  /* filename of crop in row.              */
   double             *wpolygon;  /* Array of WCS polygon vertices.        */
   double             *ipolygon;  /* Array of image polygon vertices.      */
   size_t             nvertices;  /* Number of polygon vertices.           */
   long               iwidth[2];  /* Image mode width (in pixels).         */
-  double                   res;  /* Resolution in arcseconds              */
+  double             *pixscale;  /* Resolution in each dimension.         */
   time_t               rawtime;  /* Starting time of the program.         */
   int            outnameisfile;  /* Output filename is a directory.       */
   int                     type;  /* Type of output(s).                    */
