@@ -136,7 +136,7 @@ struct mkcatalogparams
 {
   /* From command-line */
   struct gal_options_common_params cp; /* Common parameters.            */
-  gal_list_i32_t   *columnids; /* The desired column codes.   */
+  gal_list_i32_t   *columnids;  /* The desired column codes.            */
   char             *inputname;  /* Input filename.                      */
   char           *objectsfile;  /* File name of objects file.           */
   char            *objectshdu;  /* HDU of objects image.                */
@@ -156,6 +156,7 @@ struct mkcatalogparams
   char            *upmaskfile;  /* Name of upper limit mask file.       */
   char             *upmaskhdu;  /* HDU of upper limit mask file.        */
   size_t                upnum;  /* Number of upper-limit random samples.*/
+  size_t             *uprange;  /* Range of random positions about target. */
   uint8_t             envseed;  /* Use the environment for random seed. */
   double       upsigmaclip[2];  /* Sigma clip to measure upper limit.   */
   float              upnsigma;  /* Multiple of sigma to define up-lim.  */
@@ -187,13 +188,17 @@ struct mkcatalogparams
   gsl_rng                *rng;  /* Main random number generator.        */
   uint64_t               seed;  /* Random number generator seed.        */
   const char         *rngname;  /* Name of random number generator.     */
+  size_t               rngmin;  /* Minimum possible value of RNG.       */
+  size_t              rngdiff;  /* Difference of RNG max and min.       */
 
-  double              **rd_vo;  /* Object RA-Dec flux weighted X, Y.    */
-  double              **rd_vc;  /* Clump RA-Dec flux weighted X, Y.     */
-  double              **rd_go;  /* Object RA-Dec geometric X,Y.         */
-  double              **rd_gc;  /* Clump RA-Dec geometric X, Y.         */
-  double             **rd_vcc;  /* All clumps RA-Dec flx. wht. X, Y.    */
-  double             **rd_gcc;  /* All clumps RA-Dec geometric X, Y.    */
+  gal_data_t          *wcs_vo;  /* Object RA-Dec flux weighted X, Y.    */
+  gal_data_t          *wcs_vc;  /* Clump RA-Dec flux weighted X, Y.     */
+  gal_data_t          *wcs_go;  /* Object RA-Dec geometric X,Y.         */
+  gal_data_t          *wcs_gc;  /* Clump RA-Dec geometric X, Y.         */
+  gal_data_t         *wcs_vcc;  /* All clumps RA-Dec flx. wht. X, Y.    */
+  gal_data_t         *wcs_gcc;  /* All clumps RA-Dec geometric X, Y.    */
+
+  char                **ctype;  /* Type of WCS axis.                    */
 
   uint8_t            hasblank;  /* Dataset has blank values.            */
   uint8_t              hasmag;  /* Catalog has magnitude columns.       */
