@@ -29,7 +29,6 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 #include <string.h>
 #include <inttypes.h>
 
-#include <gnuastro/wcs.h>
 #include <gnuastro/fits.h>
 #include <gnuastro/blank.h>
 #include <gnuastro/threads.h>
@@ -77,12 +76,8 @@ doc[] = GAL_STRINGS_TOP_HELP_INFO PROGRAM_NAME" will create a catalog from "
   "\v"
   PACKAGE_NAME" home page: "PACKAGE_URL;
 
-/* Option groups particular to this program. */
-enum program_args_groups
-{
-  ARGS_GROUP_UPPERLIMIT = GAL_OPTIONS_GROUP_AFTER_COMMON,
-  ARGS_GROUP_COLUMNS,
-};
+
+
 
 
 
@@ -138,6 +133,7 @@ ui_initialize_options(struct mkcatalogparams *p,
       /* Select individually. */
       switch(cp->coptions[i].key)
         {
+        case GAL_OPTIONS_KEY_LOG:
         case GAL_OPTIONS_KEY_TYPE:
         case GAL_OPTIONS_KEY_SEARCHIN:
         case GAL_OPTIONS_KEY_IGNORECASE:
@@ -325,6 +321,42 @@ ui_check_options_and_arguments(struct mkcatalogparams *p)
 /**************************************************************/
 /***************       Preparations         *******************/
 /**************************************************************/
+/* If a WCS structure is present, then read its basic information to use in
+   the table meta-data. */
+static void
+ui_wcs_info(struct mkcatalogparams *p)
+{
+  char *c;
+  size_t i;
+
+  /* Read the basic WCS information. */
+  if(p->input->wcs)
+    {
+      /* Allocate space for the array of strings. */
+      errno=0;
+      p->ctype=malloc(p->input->ndim * sizeof *p->ctype);
+      if(p->ctype==NULL)
+        error(EXIT_FAILURE, 0, "%s: %zu bytes for `p->ctype'", __func__,
+              p->input->ndim * sizeof *p->ctype);
+
+      /* Fill in the values. */
+      for(i=0;i<p->input->ndim;++i)
+        {
+          /* CTYPE might contain `-' characters, we just want the first
+             non-dash characters. The loop will either stop either at the end
+             or where there is a dash. So we can just replace it with an
+             end-of-string character. */
+          gal_checkset_allocate_copy(p->input->wcs->ctype[i], &p->ctype[i]);
+          c=p->ctype[i]; while(*c!='\0' && *c!='-') ++c;
+          *c='\0';
+        }
+    }
+}
+
+
+
+
+
 static void
 ui_preparations_read_inputs(struct mkcatalogparams *p)
 {
@@ -340,12 +372,16 @@ ui_preparations_read_inputs(struct mkcatalogparams *p)
 
   /* Read the input image. */
   p->input=gal_fits_img_read_to_type(p->inputname, p->cp.hdu,
-                                     GAL_TYPE_FLOAT32, p->cp.minmapsize);
+                                     GAL_TYPE_FLOAT32, p->cp.minmapsize, 0,0);
+
+
+  /* Read basic WCS information for final table meta-data. */
+  ui_wcs_info(p);
 
 
   /* Currently MakeCatalog is only implemented for 2D images. */
   if(p->input->ndim!=2)
-    error(EXIT_FAILURE, 0, "%s (%s) has %zu dimensions, MakeCatalog "
+    error(EXIT_FAILURE, 0, "%s (hdu %s) has %zu dimensions, MakeCatalog "
           "currently only supports 2D inputs", p->inputname, p->cp.hdu,
           p->input->ndim);
 
@@ -356,7 +392,7 @@ ui_preparations_read_inputs(struct mkcatalogparams *p)
 
   /* Read the object label image and check its size. */
   p->objects = gal_fits_img_read(objectsfile, p->objectshdu,
-                                 p->cp.minmapsize);
+                                 p->cp.minmapsize, 0, 0);
   if( gal_data_dsize_is_different(p->input, p->objects) )
     error(EXIT_FAILURE, 0, "`%s' (hdu: %s) and `%s' (hdu: %s) have a"
           "different dimension/size", objectsfile, p->objectshdu,
@@ -365,7 +401,7 @@ ui_preparations_read_inputs(struct mkcatalogparams *p)
 
   /* Read the Sky image and check its size. */
   p->sky=gal_fits_img_read_to_type(skyfile, p->skyhdu, GAL_TYPE_FLOAT32,
-                                   p->cp.minmapsize);
+                                   p->cp.minmapsize, 0, 0);
   if( gal_data_dsize_is_different(p->input, p->sky) )
     error(EXIT_FAILURE, 0, "`%s' (hdu: %s) and `%s' (hdu: %s) have a"
           "different dimension/size", skyfile, p->skyhdu, p->inputname,
@@ -374,7 +410,7 @@ ui_preparations_read_inputs(struct mkcatalogparams *p)
 
   /* Read the Sky standard deviation image and check its size. */
   p->std=gal_fits_img_read_to_type(stdfile, p->stdhdu, GAL_TYPE_FLOAT32,
-                                   p->cp.minmapsize);
+                                   p->cp.minmapsize, 0, 0);
   if( gal_data_dsize_is_different(p->input, p->std) )
     error(EXIT_FAILURE, 0, "`%s' (hdu: %s) and `%s' (hdu: %s) have a"
           "different dimension/size", stdfile, p->stdhdu, p->inputname,
@@ -386,7 +422,7 @@ ui_preparations_read_inputs(struct mkcatalogparams *p)
     {
       /* Read the mask image. */
       p->upmask = gal_fits_img_read(p->upmaskfile, p->upmaskhdu,
-                                    p->cp.minmapsize);
+                                    p->cp.minmapsize, 0, 0);
       if( gal_data_dsize_is_different(p->input, p->upmask) )
         error(EXIT_FAILURE, 0, "`%s' (hdu: %s) and `%s' (hdu: %s) have a"
               "different dimension/size", p->upmaskfile, p->upmaskhdu,
@@ -442,7 +478,7 @@ ui_preparations_read_inputs(struct mkcatalogparams *p)
 
       /* Read the clumps image and check its size. */
       p->clumps = gal_fits_img_read(clumpsfile, p->clumpshdu,
-                                    p->cp.minmapsize);
+                                    p->cp.minmapsize, 0, 0);
       if( gal_data_dsize_is_different(p->input, p->std) )
         error(EXIT_FAILURE, 0, "`%s' (hdu: %s) and `%s' (hdu: %s) have a"
               "different dimension/size", clumpsfile, p->clumpshdu,
@@ -481,12 +517,6 @@ ui_preparations_read_inputs(struct mkcatalogparams *p)
     p->clumps=gal_data_copy_to_new_type_free(p->clumps, GAL_TYPE_INT32);
 
 
-  /* Read the WCS structure of the input dataset. Even if no WCS-related
-     columns are requested, we still need it to report the pixel area of
-     the input dataset. */
-  p->input->wcs=gal_wcs_read(p->inputname, p->cp.hdu, 0, 0, &p->input->nwcs);
-
-
   /* Clean up. */
   key->name=NULL;
   gal_data_array_free(key, 1, 1);
@@ -500,6 +530,7 @@ ui_preparations_read_inputs(struct mkcatalogparams *p)
 static void
 ui_preparations_read_keywords(struct mkcatalogparams *p)
 {
+  char *msg;
   gal_data_t *tmp;
   gal_data_t *keys=gal_data_array_calloc(2);
   char *stdfile=p->stdfile ? p->stdfile : p->inputname;
@@ -575,8 +606,11 @@ ui_preparations_read_keywords(struct mkcatalogparams *p)
       gal_fits_key_read(clumpsfile, p->clumpshdu, keys, 0, 0);
       if(keys[0].status) p->clumpsn=NAN;
       if(keys[1].status)
-        error(EXIT_FAILURE, 0, "couldn't find NCLUMPS in the header of "
-              "%s (hdu: %s).", p->clumpsfile, p->clumpshdu);
+        {
+          asprintf(&msg, "couldn't find/read NUMLABS in the header of "
+                   "%s (hdu: %s), see error above", clumpsfile, p->clumpshdu);
+          gal_fits_io_error(keys[1].status, msg);
+        }
     }
 
 
@@ -674,41 +708,63 @@ ui_one_tile_per_object(struct mkcatalogparams *p)
 static void
 ui_preparations_both_names(struct mkcatalogparams *p)
 {
-  char *basename;
-  char *end, *tend, suffix[50];
+  char *basename, *suffix=".fits";
   uint8_t keepinputdir=p->cp.keepinputdir;  /* See below. */
 
   /* Set the type ending. */
   if(p->cp.output)
     {
       /* When the user has specified a name, any possible directories in
-         that name must be respected. So we will keep the actual
-         `keepinputdir' value in a temporary variable, set it to 1 only for
-         this operation, then set it back to what it was. */
+         that name must be respected. So we have kept the actual
+         `keepinputdir' value in a temporary variable above and set it to 1
+         only for this operation. Later we set it back to what it was. */
       p->cp.keepinputdir=1;
 
-      /* Set the basic strings. */
+      /* Set the base name (if necessary). */
       basename = p->cp.output;
-      tend = gal_fits_name_is_fits(p->cp.output)     ? "fits" : "txt";
+
+      /* FITS speicifc preparations. */
+      if( gal_fits_name_is_fits(p->cp.output) )
+        {
+          /* The output file name that the user has given supersedes the
+             `tableformat' argument. In this case, the filename is a FITS
+             file, so if `tableformat' is a text file, we will change it to
+             a default binary FITS table. */
+          if( p->cp.tableformat==GAL_TABLE_FORMAT_TXT )
+            p->cp.tableformat=GAL_TABLE_FORMAT_BFITS;
+        }
     }
   else
     {
+      /* Note that suffix is not used in the text table outputs, so it
+         doesn't matter if the output table is not FITS. */
+      suffix="_catalog.fits";
       basename = p->inputname;
-      tend = p->cp.tableformat==GAL_TABLE_FORMAT_TXT ? "txt"  : "fits";
     }
 
-  /* Set the objects name */
-  end="_o";
-  sprintf(suffix, "%s.%s", end, tend);
-  p->objectsout=gal_checkset_automatic_output(&p->cp, basename, suffix);
 
-  /* Set the clumps name */
-  end="_c";
-  sprintf(suffix, "%s.%s", end, tend);
-  p->clumpsout=gal_checkset_automatic_output(&p->cp, basename, suffix);
+  /* Set the final filename. If the output is a text file, we need two
+     files. But when its a FITS file we want to make a multi-extension FITS
+     file. */
+  if(p->cp.tableformat==GAL_TABLE_FORMAT_TXT)
+    {
+      p->objectsout=gal_checkset_automatic_output(&p->cp, basename, "_o.txt");
+      p->clumpsout=gal_checkset_automatic_output(&p->cp, basename, "_c.txt");
+    }
+  else
+    {
+      p->objectsout=gal_checkset_automatic_output(&p->cp, basename, suffix);
+      p->clumpsout=p->objectsout;
+    }
 
-  /* Revert `keepinputdir' to what it was. */
-  if(p->cp.output) p->cp.keepinputdir=keepinputdir;
+  /* Revert `keepinputdir' to what it was and free `p->cp.output', we will
+     be using `p->objectsout' and `p->clumpsout' from now on. */
+  if(p->cp.output)
+    {
+      p->cp.keepinputdir=keepinputdir;
+      free(p->cp.output);
+      p->cp.output=NULL;
+    }
 }
 
 
@@ -737,7 +793,12 @@ ui_preparations_outnames(struct mkcatalogparams *p)
 
       /* If a clumps image has been read, then we have two outputs. */
       if(p->clumps) ui_preparations_both_names(p);
-      else          p->objectsout=p->cp.output;
+      else
+        {
+          gal_checkset_writable_remove(p->cp.output, 0, p->cp.dontdelete);
+          p->objectsout=p->cp.output;
+          p->cp.output=NULL;
+        }
     }
   else
     {
@@ -763,6 +824,18 @@ ui_preparations_outnames(struct mkcatalogparams *p)
 static void
 ui_preparations_upperlimit(struct mkcatalogparams *p)
 {
+  size_t i, c=0;
+
+  /* Check if the given range has the same number of elements as dimensions
+     in the input. */
+  if(p->uprange)
+    {
+      for(i=0;p->uprange[i]!=-1;++i) ++c;
+      if(c!=p->input->ndim)
+        error(EXIT_FAILURE, 0, "%zu values given to `--uprange', but input "
+              "has %zu dimensions", c, p->input->ndim);
+    }
+
   /* Check the number of random samples. */
   if( p->upnum < MKCATALOG_UPPERLIMIT_MINIMUM_NUM )
     error(EXIT_FAILURE, 0, "%zu not acceptable as `--upnum'. The minimum "
@@ -785,12 +858,16 @@ ui_preparations_upperlimit(struct mkcatalogparams *p)
 
   /* Set the random number generator. */
   gsl_rng_env_setup();
-  p->rng=gsl_rng_alloc(gsl_rng_default);
+  p->rng=gsl_rng_alloc(gsl_rng_ranlxs1);
   p->seed = ( p->envseed
               ? gsl_rng_default_seed
               : gal_timing_time_based_rng_seed() );
   if(p->envseed) gsl_rng_set(p->rng, p->seed);
   p->rngname=gsl_rng_name(p->rng);
+
+  /* Keep the minimum and maximum values of the random number generator. */
+  p->rngmin=gsl_rng_min(p->rng);
+  p->rngdiff=gsl_rng_max(p->rng)-p->rngmin;
 }
 
 
@@ -973,32 +1050,30 @@ ui_free_report(struct mkcatalogparams *p, struct timeval *t1)
 {
   size_t d;
 
-  /* The temporary arrays for RA and Dec. */
-  if(p->rd_vo || p->rd_vc || p->rd_go || p->rd_gc || p->rd_vcc || p->rd_gcc)
-    {
-      /* First free the separate dimensions in each value. */
-      for(d=0;d<p->input->ndim;++d)
-        {
-          if(p->rd_vo)  free(p->rd_vo[d]);
-          if(p->rd_vc)  free(p->rd_vc[d]);
-          if(p->rd_go)  free(p->rd_go[d]);
-          if(p->rd_gc)  free(p->rd_gc[d]);
-          if(p->rd_vcc) free(p->rd_vcc[d]);
-          if(p->rd_gcc) free(p->rd_gcc[d]);
-        }
+  /* The temporary arrays for WCS coordinates. */
+  if(p->wcs_vo ) gal_list_data_free(p->wcs_vo);
+  if(p->wcs_vc ) gal_list_data_free(p->wcs_vc);
+  if(p->wcs_go ) gal_list_data_free(p->wcs_go);
+  if(p->wcs_gc ) gal_list_data_free(p->wcs_gc);
+  if(p->wcs_vcc) gal_list_data_free(p->wcs_vcc);
+  if(p->wcs_gcc) gal_list_data_free(p->wcs_gcc);
 
-      /* Then free the container arrays. */
-      if(p->rd_vo)  free(p->rd_vo);
-      if(p->rd_vc)  free(p->rd_vc);
-      if(p->rd_go)  free(p->rd_go);
-      if(p->rd_gc)  free(p->rd_gc);
-      if(p->rd_vcc) free(p->rd_vcc);
-      if(p->rd_gcc) free(p->rd_gcc);
+  /* Free the types of the WCS coordinates (for catalog meta-data). */
+  if(p->ctype)
+    {
+      for(d=0;d<p->input->ndim;++d)
+        free(p->ctype[d]);
+      free(p->ctype);
     }
 
   /* If a random number generator was allocated, free it. */
   if(p->rng) gsl_rng_free(p->rng);
 
+  /* Free output names. */
+  if(p->clumpsout && p->clumpsout!=p->objectsout)
+    free(p->clumpsout);
+  free(p->objectsout);
+
   /* Free the allocated arrays: */
   free(p->skyhdu);
   free(p->stdhdu);
@@ -1007,7 +1082,6 @@ ui_free_report(struct mkcatalogparams *p, struct timeval *t1)
   free(p->ciflag);
   free(p->skyfile);
   free(p->stdfile);
-  free(p->cp.output);
   free(p->clumpshdu);
   free(p->objectshdu);
   free(p->clumpsfile);
