@@ -69,36 +69,38 @@ struct argp_option program_options[] =
       GAL_OPTIONS_NOT_MANDATORY,
       GAL_OPTIONS_NOT_SET
     },
-
-
-
-
-
     {
-      "naxis1",
-      UI_KEY_NAXIS1,
-      "INT",
+      "kernel",
+      UI_KEY_KERNEL,
+      "STR",
       0,
-      "Number of pixels along first FITS axis.",
-      GAL_OPTIONS_GROUP_OUTPUT,
-      &p->naxes[0],
-      GAL_TYPE_UINT64,
-      GAL_OPTIONS_RANGE_GT_0,
+      "Parameters to only build one kernel.",
+      GAL_OPTIONS_GROUP_INPUT,
+      &p->kernel,
+      GAL_TYPE_STRING,
+      GAL_OPTIONS_RANGE_ANY,
       GAL_OPTIONS_NOT_MANDATORY,
-      GAL_OPTIONS_NOT_SET
+      GAL_OPTIONS_NOT_SET,
+      ui_parse_kernel
     },
+
+
+
+
+
     {
-      "naxis2",
-      UI_KEY_NAXIS2,
-      "INT",
+      "naxis",
+      UI_KEY_NAXIS,
+      "INT[,INT,...]",
       0,
-      "Number of pixels along second FITS axis.",
+      "Merged image size along each dimension.",
       GAL_OPTIONS_GROUP_OUTPUT,
-      &p->naxes[1],
-      GAL_TYPE_INT64,
-      GAL_OPTIONS_RANGE_GT_0,
+      &p->dsize,
+      GAL_TYPE_STRING,
+      GAL_OPTIONS_RANGE_ANY,
       GAL_OPTIONS_NOT_MANDATORY,
-      GAL_OPTIONS_NOT_SET
+      GAL_OPTIONS_NOT_SET,
+      gal_options_parse_sizes_reverse
     },
     {
       "oversample",
@@ -160,7 +162,21 @@ struct argp_option program_options[] =
     {
       0, 0, 0, 0,
       "Profiles:",
-      ARGS_GROUP_PROFILES
+      UI_GROUP_PROFILES
+    },
+    {
+      "mode",
+      UI_KEY_MODE,
+      "STR",
+      0,
+      "Mode of `--ccol': `img' or `wcs'.",
+      UI_GROUP_PROFILES,
+      &p->mode,
+      GAL_TYPE_STRING,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_MANDATORY,
+      GAL_OPTIONS_NOT_SET,
+      ui_parse_coordinate_mode
     },
     {
       "numrandom",
@@ -168,20 +184,33 @@ struct argp_option program_options[] =
       "INT",
       0,
       "No. of random points in Monte Carlo integration.",
-      ARGS_GROUP_PROFILES,
+      UI_GROUP_PROFILES,
       &p->numrandom,
       GAL_TYPE_SIZE_T,
       GAL_OPTIONS_RANGE_GT_0,
       GAL_OPTIONS_MANDATORY,
       GAL_OPTIONS_NOT_SET
     },
+    {
+      "envseed",
+      UI_KEY_ENVSEED,
+      0,
+      0,
+      "Use GSL_RNG_SEED environment variable for seed.",
+      UI_GROUP_PROFILES,
+      &p->envseed,
+      GAL_OPTIONS_NO_ARG_TYPE,
+      GAL_OPTIONS_RANGE_0_OR_1,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
     {
       "tolerance",
       UI_KEY_TOLERANCE,
       "FLT",
       0,
       "Tolerance to switch to less accurate method.",
-      ARGS_GROUP_PROFILES,
+      UI_GROUP_PROFILES,
       &p->tolerance,
       GAL_TYPE_FLOAT32,
       GAL_OPTIONS_RANGE_GE_0_LE_1,
@@ -194,7 +223,7 @@ struct argp_option program_options[] =
       0,
       0,
       "Truncation is in units of pixels, not radius.",
-      ARGS_GROUP_PROFILES,
+      UI_GROUP_PROFILES,
       &p->tunitinp,
       GAL_OPTIONS_NO_ARG_TYPE,
       GAL_OPTIONS_RANGE_0_OR_1,
@@ -202,38 +231,52 @@ struct argp_option program_options[] =
       GAL_OPTIONS_NOT_SET
     },
     {
-      "xshift",
-      UI_KEY_XSHIFT,
-      "FLT",
+      "mforflatpix",
+      UI_KEY_MFORFLATPIX,
       0,
-      "Shift profile centers and enlarge image, X axis.",
-      ARGS_GROUP_PROFILES,
-      &p->shift[0],
-      GAL_TYPE_INT64,
-      GAL_OPTIONS_RANGE_GE_0,
+      0,
+      "mcol is flat pixel value (when fcol is 5 or 6)",
+      UI_GROUP_PROFILES,
+      &p->mforflatpix,
+      GAL_OPTIONS_NO_ARG_TYPE,
+      GAL_OPTIONS_RANGE_0_OR_1,
       GAL_OPTIONS_NOT_MANDATORY,
       GAL_OPTIONS_NOT_SET
     },
     {
-      "yshift",
-      UI_KEY_YSHIFT,
-      "FLT",
+      "mcolisbrightness",
+      UI_KEY_MCOLISBRIGHTNESS,
       0,
-      "Shift profile centers and enlarge image, Y axis.",
-      ARGS_GROUP_PROFILES,
-      &p->shift[1],
-      GAL_TYPE_INT64,
-      GAL_OPTIONS_RANGE_GE_0,
+      0,
+      "mcol is total brightness, not magnitude.",
+      UI_GROUP_PROFILES,
+      &p->mcolisbrightness,
+      GAL_OPTIONS_NO_ARG_TYPE,
+      GAL_OPTIONS_RANGE_0_OR_1,
       GAL_OPTIONS_NOT_MANDATORY,
       GAL_OPTIONS_NOT_SET
     },
+    {
+      "shift",
+      UI_KEY_SHIFT,
+      "INT[, ...]",
+      0,
+      "Shift profile centers in output image.",
+      UI_GROUP_PROFILES,
+      &p->shift,
+      GAL_TYPE_STRING,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET,
+      gal_options_parse_sizes_reverse
+    },
     {
       "prepforconv",
       UI_KEY_PREPFORCONV,
       0,
       0,
       "Shift and expand based on first catalog PSF.",
-      ARGS_GROUP_PROFILES,
+      UI_GROUP_PROFILES,
       &p->prepforconv,
       GAL_OPTIONS_NO_ARG_TYPE,
       GAL_OPTIONS_RANGE_0_OR_1,
@@ -246,11 +289,24 @@ struct argp_option program_options[] =
       "FLT",
       0,
       "Magnitude zero point.",
-      ARGS_GROUP_PROFILES,
+      UI_GROUP_PROFILES,
       &p->zeropoint,
       GAL_TYPE_FLOAT32,
       GAL_OPTIONS_RANGE_ANY,
-      GAL_OPTIONS_MANDATORY,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
+    {
+      "magatpeak",
+      UI_KEY_MAGATPEAK,
+      0,
+      0,
+      "Magnitude is for peak pixel, not full profile.",
+      UI_GROUP_PROFILES,
+      &p->magatpeak,
+      GAL_OPTIONS_NO_ARG_TYPE,
+      GAL_OPTIONS_RANGE_0_OR_1,
+      GAL_OPTIONS_NOT_MANDATORY,
       GAL_OPTIONS_NOT_SET
     },
     {
@@ -259,7 +315,7 @@ struct argp_option program_options[] =
       "FLT",
       0,
       "Width of circumference (inward) profiles",
-      ARGS_GROUP_PROFILES,
+      UI_GROUP_PROFILES,
       &p->circumwidth,
       GAL_TYPE_FLOAT32,
       GAL_OPTIONS_RANGE_GT_0,
@@ -272,39 +328,13 @@ struct argp_option program_options[] =
       0,
       0,
       "Replace overlapping profile pixels, don't add.",
-      ARGS_GROUP_PROFILES,
+      UI_GROUP_PROFILES,
       &p->replace,
       GAL_OPTIONS_NO_ARG_TYPE,
       GAL_OPTIONS_RANGE_0_OR_1,
       GAL_OPTIONS_NOT_MANDATORY,
       GAL_OPTIONS_NOT_SET
     },
-    {
-      "magatpeak",
-      UI_KEY_MAGATPEAK,
-      0,
-      0,
-      "Magnitude is for peak pixel, not full profile.",
-      ARGS_GROUP_PROFILES,
-      &p->magatpeak,
-      GAL_OPTIONS_NO_ARG_TYPE,
-      GAL_OPTIONS_RANGE_0_OR_1,
-      GAL_OPTIONS_NOT_MANDATORY,
-      GAL_OPTIONS_NOT_SET
-    },
-    {
-      "envseed",
-      UI_KEY_ENVSEED,
-      0,
-      0,
-      "Use GSL_RNG_SEED environment variable for seed.",
-      ARGS_GROUP_PROFILES,
-      &p->envseed,
-      GAL_OPTIONS_NO_ARG_TYPE,
-      GAL_OPTIONS_RANGE_0_OR_1,
-      GAL_OPTIONS_NOT_MANDATORY,
-      GAL_OPTIONS_NOT_SET
-    },
 
 
 
@@ -313,56 +343,17 @@ struct argp_option program_options[] =
     {
       0, 0, 0, 0,
       "Columns, by info (see `--searchin'), or number (starting from 1):",
-      ARGS_GROUP_CATALOG
+      UI_GROUP_CATALOG
     },
     {
-      "xcol",
-      UI_KEY_XCOL,
+      "ccol",
+      UI_KEY_CCOL,
       "STR/INT",
       0,
-      "Center along first FITS axis (horizontal).",
-      ARGS_GROUP_CATALOG,
-      &p->xcol,
-      GAL_TYPE_STRING,
-      GAL_OPTIONS_RANGE_ANY,
-      GAL_OPTIONS_NOT_MANDATORY,
-      GAL_OPTIONS_NOT_SET
-    },
-    {
-      "ycol",
-      UI_KEY_YCOL,
-      "STR/INT",
-      0,
-      "Center along second FITS axis (vertical).",
-      ARGS_GROUP_CATALOG,
-      &p->ycol,
-      GAL_TYPE_STRING,
-      GAL_OPTIONS_RANGE_ANY,
-      GAL_OPTIONS_NOT_MANDATORY,
-      GAL_OPTIONS_NOT_SET
-    },
-    {
-      "racol",
-      UI_KEY_RACOL,
-      "STR/INT",
-      0,
-      "Center right ascension.",
-      ARGS_GROUP_CATALOG,
-      &p->racol,
-      GAL_TYPE_STRING,
-      GAL_OPTIONS_RANGE_ANY,
-      GAL_OPTIONS_NOT_MANDATORY,
-      GAL_OPTIONS_NOT_SET
-    },
-    {
-      "deccol",
-      UI_KEY_DECCOL,
-      "STR/INT",
-      0,
-      "Center declination.",
-      ARGS_GROUP_CATALOG,
-      &p->deccol,
-      GAL_TYPE_STRING,
+      "Coordinate columns (one call for each dimension).",
+      UI_GROUP_CATALOG,
+      &p->ccol,
+      GAL_TYPE_STRLL,
       GAL_OPTIONS_RANGE_ANY,
       GAL_OPTIONS_NOT_MANDATORY,
       GAL_OPTIONS_NOT_SET
@@ -373,8 +364,8 @@ struct argp_option program_options[] =
       "STR/INT",
       0,
       "sersic (1), moffat (2), gaussian (3), point (4), "
-      "flat (5), circumference (6).",
-      ARGS_GROUP_CATALOG,
+      "flat (5), circumference (6), distance (7).",
+      UI_GROUP_CATALOG,
       &p->fcol,
       GAL_TYPE_STRING,
       GAL_OPTIONS_RANGE_ANY,
@@ -387,7 +378,7 @@ struct argp_option program_options[] =
       "STR/INT",
       0,
       "Effective radius or FWHM in pixels.",
-      ARGS_GROUP_CATALOG,
+      UI_GROUP_CATALOG,
       &p->rcol,
       GAL_TYPE_STRING,
       GAL_OPTIONS_RANGE_ANY,
@@ -400,7 +391,7 @@ struct argp_option program_options[] =
       "STR/INT",
       0,
       "Sersic index or Moffat beta.",
-      ARGS_GROUP_CATALOG,
+      UI_GROUP_CATALOG,
       &p->ncol,
       GAL_TYPE_STRING,
       GAL_OPTIONS_RANGE_ANY,
@@ -413,7 +404,7 @@ struct argp_option program_options[] =
       "STR/INT",
       0,
       "Position angle.",
-      ARGS_GROUP_CATALOG,
+      UI_GROUP_CATALOG,
       &p->pcol,
       GAL_TYPE_STRING,
       GAL_OPTIONS_RANGE_ANY,
@@ -426,7 +417,7 @@ struct argp_option program_options[] =
       "STR/INT",
       0,
       "Axis ratio.",
-      ARGS_GROUP_CATALOG,
+      UI_GROUP_CATALOG,
       &p->qcol,
       GAL_TYPE_STRING,
       GAL_OPTIONS_RANGE_ANY,
@@ -439,7 +430,7 @@ struct argp_option program_options[] =
       "STR/INT",
       0,
       "Magnitude.",
-      ARGS_GROUP_CATALOG,
+      UI_GROUP_CATALOG,
       &p->mcol,
       GAL_TYPE_STRING,
       GAL_OPTIONS_RANGE_ANY,
@@ -452,26 +443,13 @@ struct argp_option program_options[] =
       "STR/INT",
       0,
       "Truncation in units of --rcol, unless --tunitinp.",
-      ARGS_GROUP_CATALOG,
+      UI_GROUP_CATALOG,
       &p->tcol,
       GAL_TYPE_STRING,
       GAL_OPTIONS_RANGE_ANY,
       GAL_OPTIONS_MANDATORY,
       GAL_OPTIONS_NOT_SET
     },
-    {
-      "mforflatpix",
-      UI_KEY_MFORFLATPIX,
-      0,
-      0,
-      "mcol is flat pixel value (when fcol is 5 or 6)",
-      ARGS_GROUP_CATALOG,
-      &p->mforflatpix,
-      GAL_OPTIONS_NO_ARG_TYPE,
-      GAL_OPTIONS_RANGE_0_OR_1,
-      GAL_OPTIONS_NOT_MANDATORY,
-      GAL_OPTIONS_NOT_SET
-    },
 
 
 
@@ -480,74 +458,92 @@ struct argp_option program_options[] =
     {
       0, 0, 0, 0,
       "WCS parameters:",
-      ARGS_GROUP_WCS
+      UI_GROUP_WCS
     },
     {
-      "crpix1",
-      UI_KEY_CRPIX1,
-      "FLT",
+      "crpix",
+      UI_KEY_CRPIX,
+      "FLT[, ...]",
       0,
-      "Pixel coordinate of reference point (axis 1).",
-      ARGS_GROUP_WCS,
-      &p->crpix[0],
+      "Pixel coordinates of reference point.",
+      UI_GROUP_WCS,
+      &p->crpix,
       GAL_TYPE_FLOAT64,
       GAL_OPTIONS_RANGE_ANY,
       GAL_OPTIONS_NOT_MANDATORY,
-      GAL_OPTIONS_NOT_SET
+      GAL_OPTIONS_NOT_SET,
+      gal_options_parse_csv_float64
     },
     {
-      "crpix2",
-      UI_KEY_CRPIX2,
-      "FLT",
+      "crval",
+      UI_KEY_CRVAL,
+      "FLT[, ...]",
       0,
-      "Pixel coordinate of reference point (axis 2).",
-      ARGS_GROUP_WCS,
-      &p->crpix[1],
+      "WCS coordinates of reference point.",
+      UI_GROUP_WCS,
+      &p->crval,
       GAL_TYPE_FLOAT64,
       GAL_OPTIONS_RANGE_ANY,
       GAL_OPTIONS_NOT_MANDATORY,
-      GAL_OPTIONS_NOT_SET
+      GAL_OPTIONS_NOT_SET,
+      gal_options_parse_csv_float64
     },
     {
-      "crval1",
-      UI_KEY_CRVAL1,
-      "FLT",
+      "cdelt",
+      UI_KEY_CDELT,
+      "FLT[, ...]",
       0,
-      "Right ascension at reference point (degrees).",
-      ARGS_GROUP_WCS,
-      &p->crval[0],
+      "Resolution in each dimension.",
+      UI_GROUP_WCS,
+      &p->cdelt,
       GAL_TYPE_FLOAT64,
       GAL_OPTIONS_RANGE_ANY,
       GAL_OPTIONS_NOT_MANDATORY,
-      GAL_OPTIONS_NOT_SET
+      GAL_OPTIONS_NOT_SET,
+      gal_options_parse_csv_float64
     },
     {
-      "crval2",
-      UI_KEY_CRVAL2,
-      "FLT",
+      "pc",
+      UI_KEY_PC,
+      "FLT[, ...]",
       0,
-      "Declination at reference point (degrees).",
-      ARGS_GROUP_WCS,
-      &p->crval[1],
+      "WCS rotation matrix (all elements).",
+      UI_GROUP_WCS,
+      &p->pc,
       GAL_TYPE_FLOAT64,
       GAL_OPTIONS_RANGE_ANY,
       GAL_OPTIONS_NOT_MANDATORY,
-      GAL_OPTIONS_NOT_SET
+      GAL_OPTIONS_NOT_SET,
+      gal_options_parse_csv_float64
     },
     {
-      "resolution",
-      UI_KEY_RESOLUTION,
-      "FLT",
+      "cunit",
+      UI_KEY_CUNIT,
+      "STR[, ... ]",
       0,
-      "Resolution of image (arcseconds/pixel).",
-      ARGS_GROUP_WCS,
-      &p->resolution,
+      "Units of the WCS coordinates (e.g., `deg').",
+      UI_GROUP_WCS,
+      &p->cunit,
       GAL_TYPE_FLOAT64,
-      GAL_OPTIONS_RANGE_GT_0,
+      GAL_OPTIONS_RANGE_ANY,
       GAL_OPTIONS_NOT_MANDATORY,
-      GAL_OPTIONS_NOT_SET
+      GAL_OPTIONS_NOT_SET,
+      gal_options_parse_csv_strings
+    },
+    {
+      "ctype",
+      UI_KEY_CTYPE,
+      "STR[, ... ]",
+      0,
+      "One of FITS standard WCS types.",
+      UI_GROUP_WCS,
+      &p->ctype,
+      GAL_TYPE_FLOAT64,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET,
+      gal_options_parse_csv_strings
     },
-
 
 
 
