@@ -79,17 +79,6 @@ doc[] = GAL_STRINGS_TOP_HELP_INFO PROGRAM_NAME" will create a FITS image "
 
 
 
-/* Option groups particular to this program. */
-enum program_args_groups
-{
-  ARGS_GROUP_PROFILES = GAL_OPTIONS_GROUP_AFTER_COMMON,
-  ARGS_GROUP_CATALOG,
-  ARGS_GROUP_WCS,
-};
-
-
-
-
 
 
 
@@ -409,63 +398,6 @@ ui_parse_kernel(struct argp_option *option, char *arg,
 
 
 
-/* Parse options with values of a list of numbers. */
-void *
-ui_parse_numbers(struct argp_option *option, char *arg,
-                 char *filename, size_t lineno, void *junk)
-{
-  int i;
-  size_t nc;
-  double *darr;
-  gal_data_t *values;
-  char *str, sstr[GAL_OPTIONS_STATIC_MEM_FOR_VALUES];
-
-  /* We want to print the stored values. */
-  if(lineno==-1)
-    {
-      /* Set the pointer to the values dataset. */
-      values = *(gal_data_t **)(option->value);
-
-      /* Write each string into the output string */
-      nc=0;
-      darr=values->array;
-      for(i=0;i<values->size;++i)
-        {
-          if( nc > GAL_OPTIONS_STATIC_MEM_FOR_VALUES-100 )
-            error(EXIT_FAILURE, 0, "%s: a bug! please contact us at %s so we "
-                  "can address the problem. The number of necessary "
-                  "characters in the statically allocated string has become "
-                  "too close to %d", __func__, PACKAGE_BUGREPORT,
-                  GAL_OPTIONS_STATIC_MEM_FOR_VALUES);
-          nc += sprintf(sstr+nc, "%g,", darr[i]);
-        }
-      sstr[nc-1]='\0';
-
-      /* Copy the string into a dynamically allocated space, because it
-         will be freed later.*/
-      gal_checkset_allocate_copy(sstr, &str);
-      return str;
-    }
-
-  /* We want to read the user's string. */
-  else
-    {
-      /* If the option is already set, just return. */
-      if(option->set) return NULL;
-
-      /* Read the values. */
-      values=gal_options_parse_list_of_numbers(arg, filename, lineno);
-
-      /* Put the values into the option. */
-      *(gal_data_t **)(option->value) = values;
-      return NULL;
-    }
-}
-
-
-
-
-
 /* Parse the mode to interpret the given coordinates. */
 void *
 ui_parse_coordinate_mode(struct argp_option *option, char *arg,
@@ -688,7 +620,7 @@ ui_read_cols(struct mkprofparams *p)
 
   /* Read the desired columns from the file. */
   cols=gal_table_read(p->catname, p->cp.hdu, colstrs, p->cp.searchin,
-                      p->cp.ignorecase, p->cp.minmapsize);
+                      p->cp.ignorecase, p->cp.minmapsize, NULL);
 
   /* Set the number of objects. */
   p->num=cols->size;
@@ -1091,14 +1023,26 @@ ui_prepare_canvas(struct mkprofparams *p)
           p->ndim = *(size_t *)(keysll->array);
           keysll->name=NULL;
           gal_data_array_free(keysll, 1, 1);
+
+          /* Read the WCS structure of the background image. */
+          p->wcs=gal_wcs_read(p->backname, p->backhdu, 0, 0, &p->nwcs);
         }
       else
         {
           /* Read the image. */
           p->out=gal_fits_img_read_to_type(p->backname, p->backhdu,
                                            GAL_TYPE_FLOAT32,
-                                           p->cp.minmapsize);
+                                           p->cp.minmapsize, 0, 0);
+
+          /* Put the WCS structure and number of dimensions in the
+             MakeProfiles's main structure for generality. The WCS
+             structure will be put back in the end when writing. */
+          p->wcs=p->out->wcs;
+          p->nwcs=p->out->nwcs;
           p->ndim=p->out->ndim;
+          p->out->wcs=NULL;
+          p->out->nwcs=0;
+
 
           /* If p->dsize was given as an option, free it. */
           if( p->dsize ) free(p->dsize);
@@ -1127,9 +1071,6 @@ ui_prepare_canvas(struct mkprofparams *p)
       if(p->shift) free(p->shift);
       p->shift=gal_data_calloc_array(GAL_TYPE_SIZE_T, p->ndim, __func__,
                                      "p->shift (1)");
-
-      /* Read the WCS structure of the background image. */
-      p->wcs=gal_wcs_read(p->backname, p->backhdu, 0, 0, &p->nwcs);
     }
   else
     {
@@ -1264,9 +1205,10 @@ ui_prepare_canvas(struct mkprofparams *p)
 static void
 ui_finalize_coordinates(struct mkprofparams *p)
 {
-  size_t i, ndim=p->ndim;
-  double *x=NULL, *y=NULL;
+  void *arr=NULL;
+  size_t i=0, ndim=p->ndim;
   uint8_t os=p->oversample;
+  gal_data_t *tmp, *coords=NULL;
   double *cdelt=p->wcs->cdelt, *crpix=p->wcs->crpix;
 
   /* When the user specified RA and Dec columns, the respective values
@@ -1274,24 +1216,46 @@ ui_finalize_coordinates(struct mkprofparams *p)
      need to change them into actual image coordinates. */
   if(p->mode==MKPROF_MODE_WCS)
     {
-      /* Note that we read the RA and Dec columns into the `p->x' and `p->y'
-         arrays temporarily before. Here, we will convert them, free the old
-         ones and replace them with the proper X and Y values. */
-      gal_wcs_world_to_img(p->wcs, p->x, p->y, &x, &y, p->num);
+      /* Make list of coordinates for input of `gal_wcs_world_to_img'. */
+      for(i=0;i<ndim;++i)
+        {
+          /* Set the array pointer. Note that we read the WCS columns into
+         the `p->x', `p->y' and `p->z' arrays temporarily before. Here, we
+         will convert them to image coordinates in place. */
+          switch(i)
+            {
+            /* Note that the linked list gets filled in a first-in-last-out
+               order, so the last column added should be the first WCS
+               dimension. */
+            case 0: arr = p->y;   break;
+            case 1: arr = p->x;   break;
+            default:
+              error(EXIT_FAILURE, 0, "conversion from WCS to image "
+                    "coordinates is not currently supported for "
+                    "%zu-dimensional datasets", ndim);
+            }
+
+          /* Allocate the list of coordinates. */
+          gal_list_data_add_alloc(&coords, arr, GAL_TYPE_FLOAT64, 1, &p->num,
+                                  NULL, 0, -1, NULL, NULL, NULL);
+        }
+
+      /* Convert the world coordinates to image coordinates (inplace). */
+      gal_wcs_world_to_img(coords, p->wcs, 1);
+
 
       /* If any conversions created a WCSLIB error, both the outputs will be
          set to NaN. */
       for(i=0;i<p->num;++i)
-        if( isnan(x[i]) )
+        if( isnan(p->x[i]) )
           error(EXIT_FAILURE, 0, "catalog row %zu: WCSLIB could not convert "
                 "(%f, %f) coordinates into image coordinates", i, p->x[i],
                 p->y[i]);
 
-      /* Free the RA and Dec arrays and put in the new image values. */
-      free(p->x);
-      free(p->y);
-      p->x=x;
-      p->y=y;
+      /* We want the actual arrays of each `coords' column. So, first we'll
+         set all the array elements to NULL, then free it. */
+      for(tmp=coords;tmp!=NULL;tmp=tmp->next) tmp->array=NULL;
+      gal_list_data_free(coords);
     }
 
   /* Correct the WCS scale. Note that when the WCS is read from a
@@ -1393,7 +1357,7 @@ ui_preparations(struct mkprofparams *p)
 
   /* Allocate the random number generator: */
   gsl_rng_env_setup();
-  p->rng=gsl_rng_alloc(gsl_rng_default);
+  p->rng=gsl_rng_alloc(gsl_rng_ranlxs1);
 
   /* Make the log linked list. */
   ui_make_log(p);
