@@ -532,19 +532,37 @@ clumps_grow_prepare_final(struct clumps_thread_params *cltprm)
    This function is going to be used before identifying objects and also
    after it (to completely fill in the diffuse area). The distinguishing
    point between these two steps is the presence of rivers, so you can use
-   the `withrivers' argument. */
-void
-clumps_grow(struct clumps_thread_params *cltprm, int withrivers)
-{
-  struct noisechiselparams *p=cltprm->clprm->p;
-  gal_data_t *diffuseindexs=cltprm->diffuseindexs;
-  size_t ndim=p->input->ndim, *dsize=p->input->dsize;
+   the `withrivers' argument.
 
+
+   Input:
+
+     labels: The labels array that must be operated on. The pixels that
+             must be "grown" must have the value `CLUMPS_INIT' (negative).
+
+     diffuseindexs: The indexs of the pixels that must be grown.
+
+     withrivers: as described above.
+
+     connectivity: connectivity to define neighbors for growth.
+*/
+void
+clumps_grow(gal_data_t *labels, gal_data_t *diffuseindexs, int withrivers,
+            int connectivity)
+{
   int searchngb;
-  size_t *diarray=cltprm->diffuseindexs->array;
-  int32_t n1, nlab, *olabel=p->olabel->array;
-  size_t *dinc=gal_dimension_increment(ndim, dsize);
+  size_t *diarray=diffuseindexs->array;
+  int32_t n1, nlab, *olabel=labels->array;
   size_t *s, *sf, thisround, ndiffuse=diffuseindexs->size;
+  size_t *dinc=gal_dimension_increment(labels->ndim, labels->dsize);
+
+  /* A small sanity check: */
+  if(labels->type!=GAL_TYPE_INT32)
+    error(EXIT_FAILURE, 0, "%s: `labels' has to have type of int32_t",
+          __func__);
+  if(diffuseindexs->type!=GAL_TYPE_SIZE_T)
+    error(EXIT_FAILURE, 0, "%s: `diffuseindexs' has to have type of size_t",
+          __func__);
 
   /* The basic idea is this: after growing, not all the blank pixels are
      necessarily filled, for example the pixels might belong to two regions
@@ -580,7 +598,8 @@ clumps_grow(struct clumps_thread_params *cltprm, int withrivers)
              in a 2D image). Note that since this macro has multiple loops
              within it, we can't use break. We'll use a variable instead. */
           searchngb=1;
-          GAL_DIMENSION_NEIGHBOR_OP(*s, ndim, dsize, 1, dinc,
+          GAL_DIMENSION_NEIGHBOR_OP(*s, labels->ndim, labels->dsize,
+                                    connectivity, dinc,
             {
               if(searchngb)
                 {
@@ -813,7 +832,7 @@ clumps_make_sn_table(struct clumps_thread_params *cltprm)
 
 
   /* If there were no initial clumps, then ignore this function. */
-  if(cltprm->numinitclumps==0) { cltprm->sn=NULL; return; }
+  if(cltprm->numinitclumps==0) { cltprm->snind=cltprm->sn=NULL; return; }
 
 
   /* Allocate the arrays to keep the final S/N table (and possibly S/N
@@ -926,6 +945,11 @@ clumps_correct_sky_labels_for_check(struct clumps_thread_params *cltprm,
   size_t len=cltprm->numinitclumps+1;
   struct noisechiselparams *p=cltprm->clprm->p;
 
+  /* If there are no clumps in this tile, then this function can be
+     ignored. */
+  if(cltprm->snind->size==0) return;
+
+
   /* A small sanity check. */
   if(gal_tile_block(tile)!=p->clabel)
     error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at %s to address "
@@ -964,7 +988,6 @@ clumps_correct_sky_labels_for_check(struct clumps_thread_params *cltprm,
   do { ninds[*l]=curlab++; *l=ninds[*l]; } while(++l<lf);
 
 
-
   /* Go over this tile and correct the values. */
   GAL_TILE_PARSE_OPERATE( tile, NULL, 0, 1,
                           {if(*i>0) *i=ninds[ *(int32_t *)i ];} );
@@ -1030,7 +1053,11 @@ clumps_find_make_sn_table(void *in_prm)
 
 
       /* Find the number of detected pixels over this tile. Since this is
-         the binary image, this is just the sum of all the pixels. */
+         the binary image, this is just the sum of all the pixels.
+
+         Note that `numdet' can be `nan' when the whole tile is blank and
+         so there was no values to sum. Recall that in summing, when there
+         is not input, the output is `nan'. */
       tmp=gal_statistics_sum(tile);
       numdet=*((double *)(tmp->array));
       gal_data_free(tmp);
@@ -1049,6 +1076,7 @@ clumps_find_make_sn_table(void *in_prm)
                                        NULL, 0, p->cp.minmapsize, NULL, NULL,
                                        NULL);
 
+
           /* Change the tile's block to the clump labels dataset (because
              we'll need to set the labels of the rivers on the edge of the
              tile here). */
@@ -1398,7 +1426,7 @@ clumps_det_label_indexs(struct noisechiselparams *p)
      to allocate). */
   areas=gal_data_calloc_array(GAL_TYPE_SIZE_T, p->numdetections+1, __func__,
                               "areas");
-  if(p->input->flag & GAL_DATA_FLAG_HASBLANK)
+  if(gal_blank_present(p->input, 1))
     {
       lf=(l=p->olabel->array)+p->olabel->size; /* Blank pixels have a      */
       do if(*l>0) ++areas[*l]; while(++l<lf);  /* negative value in int32. */
