@@ -40,6 +40,7 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 
 #include "ui.h"
 #include "sky.h"
+#include "clumps.h"
 #include "threshold.h"
 
 
@@ -70,7 +71,7 @@ detection_initial(struct noisechiselparams *p)
   if(p->detectionname)
     {
       p->binary->name="THRESHOLDED";
-      gal_fits_img_write(p->binary, p->detectionname, NULL, PROGRAM_STRING);
+      gal_fits_img_write(p->binary, p->detectionname, NULL, PROGRAM_NAME);
       p->binary->name=NULL;
     }
 
@@ -88,7 +89,7 @@ detection_initial(struct noisechiselparams *p)
   if(p->detectionname)
     {
       p->binary->name="ERODED";
-      gal_fits_img_write(p->binary, p->detectionname, NULL, PROGRAM_STRING);
+      gal_fits_img_write(p->binary, p->detectionname, NULL, PROGRAM_NAME);
       p->binary->name=NULL;
     }
 
@@ -111,11 +112,12 @@ detection_initial(struct noisechiselparams *p)
 
 
   /* Label the connected components. */
-  p->numinitialdets=gal_binary_connected_components(p->binary, &p->olabel, 1);
+  p->numinitialdets=gal_binary_connected_components(p->binary, &p->olabel,
+                                                    p->binary->ndim);
   if(p->detectionname)
     {
       p->olabel->name="OPENED_AND_LABELED";
-      gal_fits_img_write(p->olabel, p->detectionname, NULL, PROGRAM_STRING);
+      gal_fits_img_write(p->olabel, p->detectionname, NULL, PROGRAM_NAME);
       p->olabel->name=NULL;
     }
 
@@ -240,8 +242,9 @@ detection_fill_holes_open(void *in_prm)
       copy->size=p->maxltcontig;
       gal_data_copy_to_allocated(tile, copy);
 
-      /* Fill the holes in this tile. */
-      gal_binary_fill_holes(copy);
+      /* Fill the holes in this tile: holes with maximal connectivity means
+         that they are most strongly bounded. */
+      gal_binary_fill_holes(copy, copy->ndim, -1);
       if(fho_prm->step==1)
         {
           detection_write_in_large(tile, copy);
@@ -291,7 +294,7 @@ detection_pseudo_find(struct noisechiselparams *p, gal_data_t *workbin,
   if(p->detectionname)
     {
       workbin->name = s0d1 ? "DTHRESH-ON-DET" : "DTHRESH-ON-SKY";
-      gal_fits_img_write(workbin, p->detectionname, NULL, PROGRAM_STRING);
+      gal_fits_img_write(workbin, p->detectionname, NULL, PROGRAM_NAME);
       workbin->name=NULL;
     }
 
@@ -346,7 +349,7 @@ detection_pseudo_find(struct noisechiselparams *p, gal_data_t *workbin,
             }
 
           /* Write the temporary array into the check image. */
-          gal_fits_img_write(bin, p->detectionname, NULL, PROGRAM_STRING);
+          gal_fits_img_write(bin, p->detectionname, NULL, PROGRAM_NAME);
 
           /* Increment the step counter. */
           ++fho_prm.step;
@@ -354,7 +357,17 @@ detection_pseudo_find(struct noisechiselparams *p, gal_data_t *workbin,
 
       /* Clean up: the array in `bin' should just be replaced with that in
          `workbin' because it is used in later steps. */
-      free(workbin->array);
+      if(workbin->mmapname)
+        {
+          /* Delete the memory mapped file and set the filename of `bin'
+             for `workbin'. */
+          remove(workbin->mmapname);
+          free(workbin->mmapname);
+          workbin->mmapname=bin->mmapname;
+          bin->mmapname=NULL;
+        }
+      else
+        free(workbin->array);
       workbin->array=bin->array;
       bin->name=bin->array=NULL;
       gal_data_free(bin);
@@ -404,7 +417,7 @@ detection_sn_write_to_file(struct noisechiselparams *p, gal_data_t *sn,
   /* Description comment. */
   str = ( s0d1D2
           ? ( s0d1D2==2
-              ? "S/N of dilated detections."
+              ? "S/N of grown detections."
               : "Pseudo-detection S/N over initial detections." )
           : "Pseudo-detection S/N over initial undetections.");
   gal_list_str_add(&comments, str, 1);
@@ -421,8 +434,8 @@ detection_sn_write_to_file(struct noisechiselparams *p, gal_data_t *sn,
   /* Abort NoiseChisel if the user asked for it. */
   if(s0d1D2==2 && !p->continueaftercheck)
     ui_abort_after_check(p, p->detsn_s_name, p->detsn_d_name,
-                         "pseudo-detection and dilated S/N values in "
-                         "a table");
+                         "pseudo-detection and grown/final detection S/N "
+                         "values in a table");
 }
 
 
@@ -547,7 +560,7 @@ detection_sn(struct noisechiselparams *p, gal_data_t *worklab, size_t num,
           while(++plab<plabend);
         }
       worklab->name=extname;
-      gal_fits_img_write(worklab, p->detectionname, NULL, PROGRAM_STRING);
+      gal_fits_img_write(worklab, p->detectionname, NULL, PROGRAM_NAME);
       worklab->name=NULL;
     }
 
@@ -659,9 +672,9 @@ detection_pseudo_remove_low_sn(struct noisechiselparams *p,
   /* If the user wanted to see the steps. */
   if(p->detectionname)
     {
-      workbin->name="TRUE-PSEUDO-DETECTIONS";
+      workbin->name="TRUE-PSEUDOS";
       gal_fits_img_write(workbin, p->detectionname, NULL,
-                         PROGRAM_STRING);
+                         PROGRAM_NAME);
       workbin->name=NULL;
     }
 
@@ -697,6 +710,17 @@ detection_pseudo_real(struct noisechiselparams *p)
   sn=detection_sn(p, worklab, numpseudo, 0, "PSEUDOS-FOR-SN");
 
 
+  /* A small sanity check */
+  if( sn->size < p->minnumfalse)
+    error(EXIT_FAILURE, 0, "only %zu pseudo-detections could be found over "
+          "the sky region to estimate an S/N. This is less than %zu (value "
+          "to `--minnumfalse' option). Please adjust parameters like "
+          "`--dthresh', `--detsnminarea', or make sure that there actually "
+          "is sufficient sky area after initial detection. You can use "
+          "`--checkdetection' to see every step until this point", sn->size,
+          p->minnumfalse);
+
+
   /* Get the S/N quantile and report it if we are in non-quiet mode. */
   quant=gal_statistics_quantile(sn, p->detquant, 1);
   p->detsnthresh = *((float *)(quant->array));
@@ -731,9 +755,10 @@ detection_pseudo_real(struct noisechiselparams *p)
 
 
 
-/* This is for the final detections (dilated) detections. */
+/* This is for the final detections (grown) detections. */
 static size_t
-detection_final_remove_small_sn(struct noisechiselparams *p, size_t num)
+detection_final_remove_small_sn(struct noisechiselparams *p,
+                                gal_data_t *workbin, size_t num)
 {
   size_t i;
   int8_t *b;
@@ -744,7 +769,6 @@ detection_final_remove_small_sn(struct noisechiselparams *p, size_t num)
   int32_t *newlabs=gal_data_calloc_array(GAL_TYPE_INT32, num+1, __func__,
                                          "newlabs");
 
-
   /* Get the Signal to noise ratio of all detections. */
   sn=detection_sn(p, p->olabel, num, 2, "DILATED");
 
@@ -756,7 +780,7 @@ detection_final_remove_small_sn(struct noisechiselparams *p, size_t num)
 
 
   /* Go over the labeled image and correct the labels. */
-  b=p->binary->array;
+  b=workbin->array;
   lf=(l=p->olabel->array)+p->olabel->size;
   if( p->input->flag & GAL_DATA_FLAG_HASBLANK )
     {
@@ -788,8 +812,7 @@ detection_final_remove_small_sn(struct noisechiselparams *p, size_t num)
       /* Make the comments, then write the table. */
       gal_list_str_add(&comments, "See also: `DILATED' "
                        "HDU of output with `--checkdetection'.", 1);
-      gal_list_str_add(&comments, "S/N of finally dilated "
-                       "detections.", 1);
+      gal_list_str_add(&comments, "S/N of finally grown detections.", 1);
 
 
       threshold_write_sn_table(p, sn, snind, p->detsn_D_name, comments);
@@ -798,15 +821,6 @@ detection_final_remove_small_sn(struct noisechiselparams *p, size_t num)
     }
 
 
-  /* For a check image. */
-  if(p->detectionname)
-    {
-      p->olabel->name="DETECTION_FINAL";
-      gal_fits_img_write(p->olabel, p->detectionname, NULL,
-                         PROGRAM_STRING);
-      p->olabel->name=NULL;
-    }
-
   /* Clean up and return. */
   free(newlabs);
   gal_data_free(sn);
@@ -876,13 +890,20 @@ detection_remove_false_initial(struct noisechiselparams *p,
   for(i=0;i<p->numinitialdets;++i) if(newlabels[i]) newlabels[i] = curlab++;
 
 
-  /* Replace the byt and olab values with their proper values. If the
-     user doesn't want to dilate, then change the labels in `lab'
-     too. Otherwise, you don't have to worry about the label
-     array. After dilation a new labeling will be done and the whole
-     labeled array will be re-filled.*/
+  /* Replace the byt and olab values with their proper values. If the user
+     doesn't want to grow, then change the labels in `lab' too. Otherwise,
+     you don't have to worry about the label array. After dilation a new
+     labeling will be done and the whole labeled array will be re-filled.*/
   b=workbin->array; l=p->olabel->array;
-  if(p->dilate)
+  if(p->detgrowquant==1.0f)          /* We need the binary array even when */
+    do                               /* there is no growth: the binary     */
+      {                              /* array is used for estimating the   */
+        if(*l!=GAL_BLANK_INT32)      /* Sky and its STD. */
+          *b = ( *l = newlabels[ *l ] ) > 0;
+        ++b;
+      }
+    while(++l<lf);
+  else
     do
       {
         if(*l!=GAL_BLANK_INT32)
@@ -890,14 +911,6 @@ detection_remove_false_initial(struct noisechiselparams *p,
         ++b;
       }
     while(++l<lf);
-  else                               /* We need the binary array even when */
-    do                               /* there is no dilation: the binary   */
-      {                              /* array is used for estimating the   */
-        if(*l!=GAL_BLANK_INT32)     /* Sky and its STD. */
-          *b = ( *l = newlabels[ *l ] ) > 0;
-        ++b;
-      }
-    while(++l<lf);
 
 
   /* Clean up and return. */
@@ -909,6 +922,98 @@ detection_remove_false_initial(struct noisechiselparams *p,
 
 
 
+static size_t
+detection_quantile_expand(struct noisechiselparams *p, gal_data_t *workbin)
+{
+  int32_t *o, *of;
+  size_t *d, counter=0, numexpanded;
+  float *i, *e_th, *arr=p->conv->array;
+  gal_data_t *exp_thresh_full, *diffuseindexs;
+  uint8_t *b=workbin->array, *bf=b+workbin->size;
+
+  /* Expand the threshold values (from one value for each tile) to the
+     whole dataset. Since we know the tiles cover the whole image, we don't
+     neeed to initialize or check for blanks.*/
+  exp_thresh_full=gal_tile_block_write_const_value(p->expand_thresh,
+                                                   p->cp.tl.tiles, 0, 0);
+
+
+  /* Count the pixels that must be expanded. */
+  e_th=exp_thresh_full->array;
+  do { if(*b++==0 && *arr>*e_th) ++counter; ++arr; ++e_th; } while(b<bf);
+
+  /* Allocate the space necessary to keep the index of all the pixels that
+     must be expanded and re-initialize the necessary pointers. */
+  diffuseindexs=gal_data_alloc(NULL, GAL_TYPE_SIZE_T, 1, &counter, NULL, 0,
+                               p->cp.minmapsize, NULL, NULL, NULL);
+
+  /* Fill in the diffuse indexs and initialize the objects dataset. */
+  b=workbin->array;
+  arr=p->conv->array;
+  d=diffuseindexs->array;
+  e_th=exp_thresh_full->array;
+  of=(o=p->olabel->array)+p->olabel->size;
+  do
+    {
+      /* If the binary value is 1, then we want an initial label of 1 (the
+         object is already detected). If it isn't, then we only want it if
+         it is above the threshold. */
+      *o = *b==1 ? 1 : ( *arr>*e_th ? CLUMPS_INIT : 0);
+      if(*b==0 && *arr>*e_th)
+        *d++ = o - (int32_t *)(p->olabel->array);
+
+      /* Increment the points and go onto the next pixel. */
+      ++b;
+      ++arr;
+      ++e_th;
+    }
+  while(++o<of);
+
+  /* Expand the detections. */
+  clumps_grow(p->olabel, diffuseindexs, 0, p->olabel->ndim);
+
+  /* Only keep the 1 valued pixels in the binary array and fill its
+     holes. */
+  o=p->olabel->array;
+  bf=(b=workbin->array)+workbin->size;
+  do *b = (*o++ == 1); while(++b<bf);
+  workbin=gal_binary_dilate(workbin, 1, 1, 1);
+  gal_binary_fill_holes(workbin, 1, p->detgrowmaxholesize);
+
+  /* Get the labeled image. */
+  numexpanded=gal_binary_connected_components(workbin, &p->olabel,
+                                              workbin->ndim);
+
+  /* Set all the input's blank pixels to blank in the labeled and binary
+     arrays. */
+  if( gal_blank_present(p->input, 1) )
+    {
+      b=workbin->array;
+      i=p->input->array;
+      of=(o=p->olabel->array)+p->olabel->size;
+      do
+        {
+          if(isnan(*i++))
+            {
+              *o=GAL_BLANK_INT32;
+              *b=GAL_BLANK_UINT8;
+            }
+          ++b;
+        }
+      while(++o<of);
+    }
+
+  /* Clean up and return */
+  gal_data_free(p->expand_thresh);
+  gal_data_free(exp_thresh_full);
+  gal_data_free(diffuseindexs);
+  return numexpanded;
+}
+
+
+
+
+
 /* The initial detection has been done, now we want to remove false
    detections. */
 void
@@ -947,11 +1052,13 @@ detection(struct noisechiselparams *p)
 
 
   /* Find the real pseudo-detections. */
+  if(!p->cp.quiet) gettimeofday(&t1, NULL);
   workbin=detection_pseudo_real(p);
 
 
   /* Only keep the initial detections that overlap with the real
      pseudo-detections. */
+  if(!p->cp.quiet) gettimeofday(&t1, NULL);
   num_true_initial=detection_remove_false_initial(p, workbin);
   if(!p->cp.quiet)
     {
@@ -962,47 +1069,46 @@ detection(struct noisechiselparams *p)
     }
 
 
-  /* If the user asked for dilation, then apply it. */
-  if(p->dilate)
-    {
-      gal_binary_dilate(workbin, p->dilate, p->input->ndim, 1);
-      num_true_initial = gal_binary_connected_components(workbin, &p->olabel,
-                                                         8);
-    }
+  /* If the user asked for dilation/expansion, then apply it and report the
+     final number of detections. */
+  if(!p->cp.quiet) gettimeofday(&t1, NULL);
+  if(p->detgrowquant!=1.0f)
+    num_true_initial=detection_quantile_expand(p, workbin);
   if(!p->cp.quiet)
     {
-      asprintf(&msg, "%zu detections after %zu dilation%s",
-              num_true_initial, p->dilate, p->dilate>1 ? "s." : ".");
-      gal_timing_report(&t0, msg, 2);
+      if(p->detgrowquant==1.0f)
+        asprintf(&msg, "%zu detections with no growth.", num_true_initial);
+      else
+        asprintf(&msg, "%zu detections after growth to %.3f quantile.",
+                 num_true_initial, p->detgrowquant);
+      gal_timing_report(&t1, msg, 2);
       free(msg);
     }
 
 
-  /* When the final (dilated or over-all object) detection's S/N is less
-     than the pseudo-detection's S/N limit, the object is false. For a real
+  /* When the final (grown or over-all object) detection's S/N is less than
+     the pseudo-detection's S/N limit, the object is false. For a real
      detection, the actual object S/N should be higher than any of its
      pseudo-detection because it has a much larger area (and possibly more
      flux under it).  So when the final S/N is smaller than the minimum
      acceptable S/N threshold, we have a false pseudo-detection. */
-  if(p->cleandilated)
-    p->numdetections = detection_final_remove_small_sn(p, num_true_initial);
-  else
-    {
-      p->numdetections = num_true_initial;
-      if(p->detectionname)
-        {
-          p->olabel->name="DETECTION_FINAL";
-          gal_fits_img_write(p->olabel, p->detectionname, NULL,
-                             PROGRAM_STRING);
-          p->olabel->name=NULL;
-        }
-    }
+  p->numdetections = ( p->cleangrowndet
+                       ?  detection_final_remove_small_sn(p, workbin,
+                                                          num_true_initial)
+                       : num_true_initial );
   if(!p->cp.quiet)
     {
       asprintf(&msg, "%zu final true detections.", p->numdetections);
       gal_timing_report(&t0, msg, 1);
       free(msg);
     }
+  if(p->detectionname)
+    {
+      p->olabel->name="DETECTION-FINAL";
+      gal_fits_img_write(p->olabel, p->detectionname, NULL,
+                         PROGRAM_NAME);
+      p->olabel->name=NULL;
+    }
 
 
   /* p->binary was used to keep the initial pseudo-detection threshold. But
