@@ -58,18 +58,48 @@ noisechisel_convolve(struct noisechiselparams *p)
   struct timeval t1;
   struct gal_tile_two_layer_params *tl=&p->cp.tl;
 
-  /* Do the convolution. */
-  if(!p->cp.quiet) gettimeofday(&t1, NULL);
-  p->conv=gal_convolve_spatial(tl->tiles, p->kernel, p->cp.numthreads,
-                               1, tl->workoverch);
-  gal_checkset_allocate_copy("INPUT_CONVOLVED", &p->conv->name);
+  /* Convovle with sharper kernel. */
+  if(p->conv==NULL)
+    {
+      /* Make the convolved image. */
+      if(!p->cp.quiet) gettimeofday(&t1, NULL);
+      p->conv = gal_convolve_spatial(tl->tiles, p->kernel, p->cp.numthreads,
+                                     1, tl->workoverch);
 
+      /* Report and write check images if necessary. */
+      if(!p->cp.quiet)
+        {
+          if(p->widekernel)
+            gal_timing_report(&t1, "Convolved with sharper kernel.", 1);
+          else
+            gal_timing_report(&t1, "Convolved with given kernel.", 1);
+        }
+    }
 
-  if(!p->cp.quiet) gal_timing_report(&t1, "Convolved with kernel.", 1);
+  /* Set a fixed name for the convolved image (since it will be used in
+     many check images). */
+  if(p->conv->name) free(p->conv->name);
+  gal_checkset_allocate_copy( ( p->widekernel
+                                ? "CONVOLVED-SHARPER"
+                                : "CONVOLVED" ), &p->conv->name);
+
+  /* Save the convolution step if necessary. */
   if(p->detectionname)
     {
-      gal_fits_img_write(p->input, p->detectionname, NULL, PROGRAM_STRING);
-      gal_fits_img_write(p->conv, p->detectionname, NULL, PROGRAM_STRING);
+      gal_fits_img_write(p->input, p->detectionname, NULL, PROGRAM_NAME);
+      gal_fits_img_write(p->conv, p->detectionname, NULL, PROGRAM_NAME);
+    }
+
+  /* Convolve with wider kernel (if requested). */
+  if(p->widekernel)
+    {
+      if(!p->cp.quiet) gettimeofday(&t1, NULL);
+      p->wconv=gal_convolve_spatial(tl->tiles, p->widekernel,
+                                    p->cp.numthreads, 1, tl->workoverch);
+      gal_checkset_allocate_copy("CONVOLVED-WIDER", &p->wconv->name);
+
+      if(!p->cp.quiet)
+        gal_timing_report(&t1, "Convolved with wider kernel.", 1);
     }
 }
 
@@ -205,7 +235,7 @@ noisechisel_output(struct noisechiselparams *p)
                         0, "Minimum S/N of true pseudo-detections", 0,
                         "ratio");
   p->olabel->name = p->onlydetection ? "DETECTIONS" : "OBJECTS";
-  gal_fits_img_write(p->olabel, p->cp.output, keys, PROGRAM_STRING);
+  gal_fits_img_write(p->olabel, p->cp.output, keys, PROGRAM_NAME);
   p->olabel->name=NULL;
   keys=NULL;
 
@@ -223,7 +253,7 @@ noisechisel_output(struct noisechiselparams *p)
       gal_fits_key_list_add(&keys, GAL_TYPE_FLOAT32, "CLUMPSN", 0,
                             &p->clumpsnthresh, 0, "Minimum S/N of true clumps",
                             0, "ratio");
-      gal_fits_img_write(p->clabel, p->cp.output, keys, PROGRAM_STRING);
+      gal_fits_img_write(p->clabel, p->cp.output, keys, PROGRAM_NAME);
       p->clabel->name=NULL;
       keys=NULL;
     }
@@ -232,8 +262,8 @@ noisechisel_output(struct noisechiselparams *p)
   /* Write the Sky image into the output */
   if(p->sky->name) free(p->sky->name);
   p->sky->name="SKY";
-  gal_tile_full_values_write(p->sky, &p->cp.tl, p->cp.output,
-                             NULL, PROGRAM_STRING);
+  gal_tile_full_values_write(p->sky, &p->cp.tl, 1, p->cp.output,
+                             NULL, PROGRAM_NAME);
   p->sky->name=NULL;
 
 
@@ -248,8 +278,8 @@ noisechisel_output(struct noisechiselparams *p)
   gal_fits_key_list_add(&keys, GAL_TYPE_FLOAT32, "MEDSTD", 0, &p->medstd, 0,
                         "Median raw tile standard deviation", 0,
                         p->input->unit);
-  gal_tile_full_values_write(p->std, &p->cp.tl, p->cp.output, keys,
-                             PROGRAM_STRING);
+  gal_tile_full_values_write(p->std, &p->cp.tl, 1, p->cp.output, keys,
+                             PROGRAM_NAME);
   p->std->name=NULL;
 }
 
@@ -298,6 +328,8 @@ noisechisel(struct noisechiselparams *p)
       /* Do the segmentation. */
       segmentation(p);
     }
+  else
+    p->numobjects=p->numdetections;
 
   /* Write the output. */
   noisechisel_output(p);
