@@ -35,6 +35,7 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 #include <gnuastro/dimension.h>
 
 #include <gnuastro-internal/timing.h>
+#include <gnuastro-internal/checkset.h>
 
 #include "main.h"
 
@@ -411,6 +412,7 @@ segmentation_on_threads(void *in_prm)
       /* Find the clumps over this region. */
       clumps_oversegment(&cltprm);
 
+
       /* Make the clump S/N table. This table is made before (possibly)
          stopping the process (if a check is requested). This is because if
          the user has also asked for a check image, we can break out of the
@@ -424,6 +426,7 @@ segmentation_on_threads(void *in_prm)
         clumps_make_sn_table(&cltprm);
       else cltprm.sn=&clprm->sn[ cltprm.id ];
 
+
       /* If the user wanted to check the segmentation steps or the clump
          S/N values in a table, then we have to stop the process at this
          point. */
@@ -435,6 +438,7 @@ segmentation_on_threads(void *in_prm)
       gal_data_free(topinds);
       if(clprm->step==2) continue;
 
+
       /* Set the internal (with the detection) clump and object
          labels. Segmenting a detection into multiple objects is only
          defined when there is more than one true clump over the
@@ -448,12 +452,10 @@ segmentation_on_threads(void *in_prm)
           cltprm.numobjects=1;
           segmentation_relab_noseg(&cltprm);
 
-
           /* If the user wanted a check image, this object doesn't
              change. */
           if( clprm->step >= 3 && clprm->step <= 6) continue;
 
-
           /* If the user has asked for grown clumps in the clumps image
              instead of the raw clumps, then replace the indexs in the
              `clabel' array is well. In this case, there will always be one
@@ -469,11 +471,11 @@ segmentation_on_threads(void *in_prm)
         {
           /* Grow the true clumps over the detection. */
           clumps_grow_prepare_initial(&cltprm);
-          if(cltprm.diffuseindexs->size) clumps_grow(&cltprm, 1);
+          if(cltprm.diffuseindexs->size)
+            clumps_grow(p->olabel, cltprm.diffuseindexs, 1, 1);
           if(clprm->step==3)
             { gal_data_free(cltprm.diffuseindexs); continue; }
 
-
           /* If grown clumps are desired instead of the raw clumps, then
              replace all the grown clumps with those in clabel. */
           if(p->grownclumps)
@@ -484,7 +486,6 @@ segmentation_on_threads(void *in_prm)
               while(++s<sf);
             }
 
-
           /* Identify the objects in this detection using the grown clumps
              and correct the grown clump labels into new object labels. */
           segmentation_relab_to_objects(&cltprm);
@@ -495,7 +496,6 @@ segmentation_on_threads(void *in_prm)
               continue;
             }
 
-
           /* Continue the growth and cover the whole area, we don't need
              the diffuse indexs any more, so after filling the detected
              region, free the indexs. */
@@ -507,13 +507,21 @@ segmentation_on_threads(void *in_prm)
                  grown. */
               clumps_grow_prepare_final(&cltprm);
 
-              /* Cover the whole area. */
-              clumps_grow(&cltprm, 0);
+              /* Cover the whole area (using maximum connectivity to not
+                 miss any pixels). */
+              clumps_grow(p->olabel, cltprm.diffuseindexs, 0,
+                          p->olabel->ndim);
+
+              /* Make sure all diffuse pixels are labeled. */
+              if(cltprm.diffuseindexs->size)
+                error(EXIT_FAILURE, 0, "a bug! Please contact us at %s to "
+                      "fix it. %zu pixels of detection %zu have not been "
+                      "labeled (as an object)", PACKAGE_BUGREPORT,
+                      cltprm.diffuseindexs->size, cltprm.id);
             }
           gal_data_free(cltprm.diffuseindexs);
           if(clprm->step==5) { gal_data_free(cltprm.clumptoobj); continue; }
 
-
           /* Correct the clump labels. Note that this is only necessary
              when there is more than object over the detection or when
              there were multiple clumps over the detection. */
@@ -595,7 +603,9 @@ segmentation_save_sn_table(struct clumps_params *clprm)
   /* Set the column pointers and write them into a table.. */
   clumpinobj->next=sn;
   objind->next=clumpinobj;
-  gal_table_write(objind, comments, p->cp.tableformat, p->clumpsn_d_name, 1);
+  gal_checkset_writable_remove(p->clumpsn_d_name, 0, 1);
+  gal_table_write(objind, comments, p->cp.tableformat, p->clumpsn_d_name,
+                  "CLUMPS_SN");
 
 
   /* Clean up. */
@@ -759,8 +769,9 @@ segmentation_detections(struct noisechiselparams *p)
 
             default:
               error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at %s so "
-                    "we can address the issue. The value %d is not recognized "
-                    "for clprm.step", __func__, PACKAGE_BUGREPORT, clprm.step);
+                    "we can address the issue. The value %d is not "
+                    "recognized for clprm.step", __func__, PACKAGE_BUGREPORT,
+                    clprm.step);
             }
 
           /* Write the demonstration array into the check image. The
@@ -790,6 +801,7 @@ segmentation_detections(struct noisechiselparams *p)
                            p->cp.numthreads);
     }
 
+
   /* Save the final number of objects and clumps. */
   p->numclumps=clprm.totclumps;
   p->numobjects=clprm.totobjects;
