@@ -23,13 +23,27 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 #ifndef UI_H
 #define UI_H
 
+/* For common options groups. */
+#include <gnuastro-internal/options.h>
+
+
+
+
+
+/* Option groups particular to this program. */
+enum program_args_groups
+{
+  UI_GROUP_DETECTION = GAL_OPTIONS_GROUP_AFTER_COMMON,
+  UI_GROUP_SEGMENTATION,
+};
+
 
 
 
 
 /* Available letters for short options:
 
-   a b f j l n u w x z
+   a b f j l n u x z
    A H J W X Y
 */
 enum option_keys_enum
@@ -37,6 +51,7 @@ enum option_keys_enum
   /* With short-option version. */
   UI_KEY_LARGETILESIZE      = 'L',
   UI_KEY_KERNEL             = 'k',
+  UI_KEY_WIDEKERNEL         = 'w',
   UI_KEY_SKYSUBTRACTED      = 'E',
   UI_KEY_MINSKYFRAC         = 'B',
   UI_KEY_MIRRORDIST         = 'r',
@@ -48,7 +63,7 @@ enum option_keys_enum
   UI_KEY_DTHRESH            = 'R',
   UI_KEY_DETSNMINAREA       = 'i',
   UI_KEY_DETQUANT           = 'c',
-  UI_KEY_DILATE             = 'd',
+  UI_KEY_DETGROWQUANT       = 'd',
   UI_KEY_SEGSNMINAREA       = 'm',
   UI_KEY_SEGQUANT           = 'g',
   UI_KEY_KEEPMAXNEARRIVER   = 'v',
@@ -61,18 +76,22 @@ enum option_keys_enum
   /* Only with long version (start with a value 1000, the rest will be set
      automatically). */
   UI_KEY_KHDU               = 1000,
+  UI_KEY_CONVOLVED,
+  UI_KEY_CONVOLVEDHDU,
+  UI_KEY_WKHDU,
   UI_KEY_MINNUMFALSE,
   UI_KEY_ONLYDETECTION,
   UI_KEY_GROWNCLUMPS,
   UI_KEY_SMOOTHWIDTH,
+  UI_KEY_QTHRESHTILEQUANT,
   UI_KEY_CHECKQTHRESH,
   UI_KEY_ERODENGB,
   UI_KEY_NOERODEQUANT,
   UI_KEY_OPENINGNGB,
   UI_KEY_CHECKDETSKY,
   UI_KEY_CHECKDETSN,
-  UI_KEY_DILATENGB,
-  UI_KEY_CLEANDILATED,
+  UI_KEY_DETGROWMAXHOLESIZE,
+  UI_KEY_CLEANGROWNDET,
   UI_KEY_CHECKDETECTION,
   UI_KEY_CHECKSKY,
   UI_KEY_CHECKCLUMPSN,
