@@ -38,7 +38,6 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 #include <gnuastro/statistics.h>
 
 #include <gnuastro-internal/timing.h>
-#include <gnuastro-internal/options.h>
 #include <gnuastro-internal/checkset.h>
 #include <gnuastro-internal/tableintern.h>
 #include <gnuastro-internal/fixedstringmacros.h>
@@ -85,14 +84,7 @@ doc[] = GAL_STRINGS_TOP_HELP_INFO PROGRAM_NAME" will do statistical "
 
 
 
-/* Option groups particular to this program. */
-enum program_args_groups
-{
-  ARGS_GROUP_SINGLE_VALUE = GAL_OPTIONS_GROUP_AFTER_COMMON,
-  ARGS_GROUP_PARTICULAR_STAT,
-  ARGS_GROUP_SKY,
-  ARGS_GROUP_HIST_CFP,
-};
+
 
 
 
@@ -225,8 +217,9 @@ ui_add_to_single_value(struct argp_option *option, char *arg,
           "(like `--number', `--mean', and etc) do not support printing "
           "with the `--printparams' (`-P'), or writing into configuration "
           "files due to lack of time when implementing these features. "
-          "Please get in touch with us at `%s', so we can implement it if "
-          "it is possible now, thank you", PACKAGE_BUGREPORT);
+          "You can put them into configuration files manually. Please get "
+          "in touch with us at `%s', so we can implement it",
+          PACKAGE_BUGREPORT);
 
   /* Some of these options take values and some don't. */
   if(option->type==GAL_OPTIONS_NO_ARG_TYPE)
@@ -726,7 +719,7 @@ ui_read_columns(struct statisticsparams *p)
 
   /* Read the desired column(s). */
   cols=gal_table_read(p->inputname, p->cp.hdu, column, p->cp.searchin,
-                      p->cp.ignorecase, p->cp.minmapsize);
+                      p->cp.ignorecase, p->cp.minmapsize, NULL);
 
   /* Put the columns into the proper gal_data_t. */
   size=cols->size;
@@ -789,10 +782,7 @@ ui_preparations(struct statisticsparams *p)
   if(p->isfits && p->hdu_type==IMAGE_HDU)
     {
       p->inputformat=INPUT_FORMAT_IMAGE;
-      p->input=gal_fits_img_read(p->inputname, cp->hdu, cp->minmapsize);
-      if(p->ontile)
-        p->input->wcs=gal_wcs_read(p->inputname, cp->hdu, 0, 0,
-                                   &p->input->nwcs);
+      p->input=gal_fits_img_read(p->inputname, cp->hdu, cp->minmapsize, 0, 0);
     }
   else
     {
@@ -822,8 +812,12 @@ ui_preparations(struct statisticsparams *p)
           if(p->inputformat==INPUT_FORMAT_IMAGE)
             gal_fits_img_write(check, tl->tilecheckname, NULL, PROGRAM_NAME);
           else
-            gal_table_write(check, NULL, cp->tableformat, tl->tilecheckname,
-                            cp->dontdelete);
+            {
+              gal_checkset_writable_remove(tl->tilecheckname, 0,
+                                           cp->dontdelete);
+              gal_table_write(check, NULL, cp->tableformat, tl->tilecheckname,
+                              "TABLE");
+            }
           gal_data_free(check);
         }
 
