@@ -75,15 +75,6 @@ doc[] = GAL_STRINGS_TOP_HELP_INFO PROGRAM_NAME" will warp/transform the "
 
 
 
-/* Option groups particular to this program. */
-enum program_args_groups
-{
-  ARGS_GROUP_WARPS = GAL_OPTIONS_GROUP_AFTER_COMMON,
-};
-
-
-
-
 
 
 
@@ -347,10 +338,8 @@ ui_check_options_and_arguments(struct warpparams *p)
 
       /* Read the input image as double type and its WCS structure. */
       p->input=gal_fits_img_read_to_type(p->inputname, p->cp.hdu,
-                                         GAL_TYPE_FLOAT64,
-                                         p->cp.minmapsize);
-      p->input->wcs=gal_wcs_read(p->inputname, p->cp.hdu, p->hstartwcs,
-                                 p->hendwcs, &p->input->nwcs);
+                                         GAL_TYPE_FLOAT64, p->cp.minmapsize,
+                                         p->hstartwcs, p->hendwcs);
       if(p->input->wcs)
         {
           p->pixelscale=gal_wcs_pixel_scale(p->input->wcs);
