@@ -51,6 +51,10 @@ __BEGIN_C_DECLS  /* From C++ preparations */
          All the axises are based on the FITS standard, NOT C.
 */
 
+void
+gal_box_bound_ellipse_extent(double a, double b, double theta_deg,
+                             double *extent);
+
 void
 gal_box_bound_ellipse(double a, double b, double theta_deg, long *width);
 
