@@ -34,7 +34,7 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 
 /* Configuration macros: */
 
-#define GAL_CONFIG_VERSION              "0.4"
+#define GAL_CONFIG_VERSION              "0.5"
 
 #define GAL_CONFIG_HAVE_LIBGIT2         1
 
@@ -42,16 +42,16 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 
 #define GAL_CONFIG_HAVE_PTHREAD_BARRIER 1
 
-#define GAL_CONFIG_BIN_OP_UINT8         1
-#define GAL_CONFIG_BIN_OP_INT8          0
-#define GAL_CONFIG_BIN_OP_UINT16        0
-#define GAL_CONFIG_BIN_OP_INT16         0
-#define GAL_CONFIG_BIN_OP_UINT32        0
-#define GAL_CONFIG_BIN_OP_INT32         0
-#define GAL_CONFIG_BIN_OP_UINT64        1
-#define GAL_CONFIG_BIN_OP_INT64         1
-#define GAL_CONFIG_BIN_OP_FLOAT32       1
-#define GAL_CONFIG_BIN_OP_FLOAT64       1
+#define GAL_CONFIG_BIN_OP_UINT8         @HAVE_BIN_OP_UINT8@
+#define GAL_CONFIG_BIN_OP_INT8          @HAVE_BIN_OP_INT8@
+#define GAL_CONFIG_BIN_OP_UINT16        @HAVE_BIN_OP_UINT16@
+#define GAL_CONFIG_BIN_OP_INT16         @HAVE_BIN_OP_INT16@
+#define GAL_CONFIG_BIN_OP_UINT32        @HAVE_BIN_OP_UINT32@
+#define GAL_CONFIG_BIN_OP_INT32         @HAVE_BIN_OP_INT32@
+#define GAL_CONFIG_BIN_OP_UINT64        @HAVE_BIN_OP_UINT64@
+#define GAL_CONFIG_BIN_OP_INT64         @HAVE_BIN_OP_INT64@
+#define GAL_CONFIG_BIN_OP_FLOAT32       @HAVE_BIN_OP_FLOAT32@
+#define GAL_CONFIG_BIN_OP_FLOAT64       @HAVE_BIN_OP_FLOAT64@
 
 #define GAL_CONFIG_SIZEOF_LONG          8
 #define GAL_CONFIG_SIZEOF_SIZE_T        8
