@@ -44,6 +44,7 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 #include <wcslib/wcshdr.h>
 #include <wcslib/wcsfix.h>
 
+#include <gnuastro/list.h>
 #include <gnuastro/data.h>
 #include <gnuastro/table.h>
 
@@ -159,6 +160,9 @@ gal_fits_hdu_open_format(char *filename, char *hdu, int img0_tab1);
 /**************************************************************/
 /**********            Header keywords             ************/
 /**************************************************************/
+void *
+gal_fits_key_img_blank(uint8_t type);
+
 void
 gal_fits_key_clean_str_value(char *string);
 
@@ -206,11 +210,13 @@ gal_fits_img_info(fitsfile *fptr, int *type, size_t *ndim, size_t **dsize,
                   char **name, char **unit);
 
 gal_data_t *
-gal_fits_img_read(char *filename, char *hdu, size_t minmapsize);
+gal_fits_img_read(char *filename, char *hdu, size_t minmapsize,
+                  size_t hstartwcs, size_t hendwcs);
 
 gal_data_t *
-gal_fits_img_read_to_type(char *inputname, char *inhdu, uint8_t type,
-                          size_t minmapsize);
+gal_fits_img_read_to_type(char *inputname, char *hdu, uint8_t type,
+                          size_t minmapsize, size_t hstartwcs,
+                          size_t hendwcs);
 
 gal_data_t *
 gal_fits_img_read_kernel(char *filename, char *hdu, size_t minmapsize);
@@ -257,7 +263,7 @@ gal_fits_tab_read(char *filename, char *hdu, size_t numrows,
 
 void
 gal_fits_tab_write(gal_data_t *cols, gal_list_str_t *comments,
-                   int tableformat, char *filename, int dontdelete);
+                   int tableformat, char *filename, char *extname);
 
 
 
