@@ -37,6 +37,7 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 #include <gnuastro/tile.h>
 #include <gnuastro/fits.h>
 #include <gnuastro/dimension.h>
+#include <gnuastro/permutation.h>
 
 
 
@@ -70,8 +71,8 @@ gal_wcs_read_fitsptr(fitsfile *fptr, size_t hstartwcs, size_t hendwcs,
                      int *nwcs)
 {
   /* Declaratins: */
-  struct wcsprm *wcs;
   int nkeys=0, status=0;
+  struct wcsprm *wcs=NULL;
   char *fullheader, *to, *from;
   int relax    = WCSHDR_all; /* Macro: use all informal WCS extensions. */
   int ctrl     = 0;          /* Don't report why a keyword wasn't used. */
@@ -111,7 +112,8 @@ gal_wcs_read_fitsptr(fitsfile *fptr, size_t hstartwcs, size_t hendwcs,
       /*******************************************************/
     }
 
-  /* WCSlib function */
+
+  /* WCSlib function to parse the FITS headers. */
   status=wcspih(fullheader, nkeys, relax, ctrl, &nreject, nwcs, &wcs);
   if(status)
     {
@@ -125,22 +127,45 @@ gal_wcs_read_fitsptr(fitsfile *fptr, size_t hstartwcs, size_t hendwcs,
     gal_fits_io_error(status, "problem in fitsarrayvv.c for freeing "
                            "the memory used to keep all the headers");
 
+
   /* Set the internal structure: */
-  status=wcsset(wcs);
-  if(status)
+  if(wcs)
     {
-      fprintf(stderr, "\n##################\n"
-              "WCSLIB Warning: wcsset ERROR %d: %s.\n"
-              "##################\n",
-            status, wcs_errmsg[status]);
-      wcs=NULL; *nwcs=0;
+      /* CTYPE is a mandatory WCS keyword, so if it hasn't been given (its
+         '\0'), then the headers didn't have a WCS structure. However,
+         WCSLIB still fills in the basic information (for example the
+         dimensionality of the dataset). */
+      if(wcs->ctype[0][0]=='\0')
+        {
+          wcsfree(wcs);
+          wcs=NULL;
+          *nwcs=0;
+        }
+      else
+        {
+          /* Set the WCS structure. */
+          status=wcsset(wcs);
+          if(status)
+            {
+              fprintf(stderr, "\n##################\n"
+                      "WCSLIB Warning: wcsset ERROR %d: %s.\n"
+                      "##################\n",
+                      status, wcs_errmsg[status]);
+              wcsfree(wcs);
+              wcs=NULL;
+              *nwcs=0;
+            }
+          else
+            /* A correctly useful WCS is present. When no PC matrix
+               elements were present in the header, the default PC matrix
+               (a unity matrix) is used. In this case WCSLIB doesn't set
+               `altlin' (and gives it a value of 0). In Gnuastro, later on,
+               we might need to know the type of the matrix used, so in
+               such a case, we will set `altlin' to 1. */
+            if(wcs->altlin==0) wcs->altlin=1;
+        }
     }
 
-  /* Initialize the wcsprm struct
-  if ((status = wcsset(*wcs)))
-    error(EXIT_FAILURE, 0, "%s: wcsset ERROR %d: %s.\n", __func__,
-          status, wcs_errmsg[status]);
-  */
 
   /* Return the WCS structure. */
   return wcs;
@@ -294,13 +319,13 @@ gal_wcs_warp_matrix(struct wcsprm *wcs)
           __func__, size*sizeof *out);
 
   /* Fill in the array. */
-  if(wcs->altlin & 0x1)        /* Has a PCi_j array. */
+  if(wcs->altlin & 0x1)          /* Has a PCi_j array. */
     {
       for(i=0;i<wcs->naxis;++i)
         for(j=0;j<wcs->naxis;++j)
           out[i*wcs->naxis+j] = wcs->cdelt[i] * wcs->pc[i*wcs->naxis+j];
     }
-  else if(wcs->altlin & 0x2)     /* Has CDi_j array */
+  else if(wcs->altlin & 0x2)     /* Has CDi_j array.   */
     {
       for(i=0;i<size;++i)
         out[i]=wcs->cd[i];
@@ -339,7 +364,7 @@ gal_wcs_decompose_pc_cdelt(struct wcsprm *wcs)
   if(wcs->pc)
     {
       /* Get the pixel scale. */
-      ps=gal_wcs_pixel_scale_deg(wcs);
+      ps=gal_wcs_pixel_scale(wcs);
 
       /* The PC matrix and the CDELT elements might both contain scale
          elements (during processing the scalings might be added only to PC
@@ -407,43 +432,135 @@ gal_wcs_angular_distance_deg(double r1, double d1, double r2, double d2)
 
 
 
-/* Return the pixel scale of the image in both dimentions in degrees. */
+/* Return the pixel scale of the dataset in units of the WCS. */
 double *
-gal_wcs_pixel_scale_deg(struct wcsprm *wcs)
+gal_wcs_pixel_scale(struct wcsprm *wcs)
 {
   gsl_vector S;
   gsl_matrix A, V;
-  size_t n=wcs->naxis;
-  double *a, *v, *pixscale;
+  int warning_printed;
+  size_t i, j, maxj, n=wcs->naxis;
+  double jvmax, *a, *out, maxrow, minrow;
+  double *v=gal_data_malloc_array(GAL_TYPE_FLOAT64, n*n, __func__, "v");
+  size_t *permutation=gal_data_malloc_array(GAL_TYPE_SIZE_T, n, __func__,
+                                            "permutation");
+  gal_data_t *pixscale=gal_data_alloc(NULL, GAL_TYPE_FLOAT64, 1, &n, NULL,
+                                      0, -1, NULL, NULL, NULL);
 
-  /* Allocate space for the `v' and `pixscale' arrays. */
-  errno=0; v=malloc(n*n*sizeof *v);
-  if(v==NULL)
-    error(EXIT_FAILURE, errno, "%s: allocating %zu bytes for `v'",
-          __func__, n*n*sizeof *v);
-  errno=0; pixscale=malloc(n*sizeof *pixscale);
-  if(pixscale==NULL)
-    error(EXIT_FAILURE, errno, "%s: allocating %zu bytes for `pixscale'",
-          __func__, n*sizeof *pixscale);
 
-  /* Write the full matrix into an array, irrespective of what type it was
-     stored in the wcsprm structure (`PCi_j' style or `CDi_j' style). */
+  /* Write the full WCS rotation matrix into an array, irrespective of what
+     style it was stored in the wcsprm structure (`PCi_j' style or `CDi_j'
+     style). */
   a=gal_wcs_warp_matrix(wcs);
 
+
+  /* To avoid confusing issues with floating point errors being written in
+     the non-diagonal elements of the FITS header PC or CD matrices, we
+     need to check if the minimum and maximum values in each row are not
+     several orders of magnitude apart.
+
+     Note that in some cases (for example a spectrum), one axis might be in
+     degrees (value around 1e-5) and the other in angestroms (value around
+     1e-10). So we can't look at the minimum and maximum of the whole
+     matrix. However, in such cases, people will probably not warp/rotate
+     the image to mix the coordinates. So the important thing to check is
+     the minimum and maximum (non-zero) values in each row. */
+  warning_printed=0;
+  for(i=0;i<n;++i)
+    {
+      /* Find the minimum and maximum values in each row. */
+      minrow=FLT_MAX;
+      maxrow=-FLT_MAX;
+      for(j=0;j<n;++j)
+        if(a[i*n+j]!=0.0) /* We aren't concerned with 0 valued elements. */
+          {
+            /* We have to use absolutes because in cases like RA, the
+               diagonal values in different rows may have different signs*/
+            if(fabs(a[i*n+j])<minrow) minrow=fabs(a[i*n+j]);
+            if(fabs(a[i*n+j])>maxrow) maxrow=fabs(a[i*n+j]);
+          }
+
+      /* Do the check, print warning and make correction. */
+      if(maxrow!=minrow && maxrow/minrow>1e4 && warning_printed==0)
+        {
+          fprintf(stderr, "\nWARNING: The input WCS matrix (possibly taken "
+                  "from the FITS header keywords starting with `CD' or `PC') "
+                  "contains values with very different scales (more than "
+                  "10^4 different). This is probably due to floating point "
+                  "errors. These values might bias the pixel scale (and "
+                  "subsequent) calculations.\n\n"
+                  "You can see the respective matrix with one of the "
+                  "following two commands (depending on how the FITS file "
+                  "was written). Recall that if the desired extension/HDU "
+                  "isn't the default, you can choose it with the `--hdu' "
+                  "(or `-h') option before the `|' sign in these commands."
+                  "\n\n"
+                  "    $ astfits file.fits -p | grep 'PC._.'\n"
+                  "    $ astfits file.fits -p | grep 'CD._.'\n\n"
+                  "You can delete the ones with obvious floating point "
+                  "error values using the following command (assuming you "
+                  "want to delete `CD1_2' and `CD2_1'). Afterwards, you can "
+                  "rerun your original command to remove this warning "
+                  "message and possibly correct errors that it might have "
+                  "caused.\n\n"
+                  "    $ astfits file.fits --delete=CD1_2 --delete=CD2_1\n\n"
+                  );
+          warning_printed=1;
+        }
+    }
+
+
   /* Fill in the necessary GSL vector and Matrix structures. */
-  S.size=n;     S.stride=1;                S.data=pixscale;
+  S.size=n;     S.stride=1;                S.data=pixscale->array;
   V.size1=n;    V.size2=n;    V.tda=n;     V.data=v;
   A.size1=n;    A.size2=n;    A.tda=n;     A.data=a;
 
+
   /* Run GSL's Singular Value Decomposition, using one-sided Jacobi
      orthogonalization which computes the singular (scale) values to a
-     higher relative accuracy.*/
+     higher relative accuracy. */
   gsl_linalg_SV_decomp_jacobi(&A, &V, &S);
 
+
+  /* The raw pixel scale array produced from the singular value
+     decomposition above is ordered based on values, not the input. So when
+     the pixel scales in all the dimensions aren't the same (the units of
+     the dimensions differ), the order of the values in `pixelscale' will
+     not necessarily correspond to the input's dimensions.
+
+     To correct the order, we can use the `V' matrix to find the original
+     position of the pixel scale values and then use permutation to
+     re-order it correspondingly. The column with the largest (absolute)
+     value will be taken as the one to be used for each row. */
+  for(i=0;i<n;++i)
+    {
+      /* Find the column with the maximum value. */
+      maxj=-1;
+      jvmax=-FLT_MAX;
+      for(j=0;j<n;++j)
+        if(fabs(v[i*n+j])>jvmax)
+          {
+            maxj=j;
+            jvmax=fabs(v[i*n+j]);
+          }
+
+      /* Use the column with the maximum value for this dimension. */
+      permutation[i]=maxj;
+    }
+
+
+  /* Apply the permutation described above. */
+  gal_permutation_apply(pixscale, permutation);
+
+
   /* Clean up and return. */
   free(a);
   free(v);
-  return pixscale;
+  free(permutation);
+  out=pixscale->array;
+  pixscale->array=NULL;
+  gal_data_free(pixscale);
+  return out;
 }
 
 
@@ -461,12 +578,16 @@ gal_wcs_pixel_area_arcsec2(struct wcsprm *wcs)
   /* A small sanity check. Later, when higher dimensions are necessary, we
      can find which ones correlate to RA and Dec and use them to find the
      pixel area in arcsec^2. */
-  if(wcs->naxis!=2)
-    error(EXIT_FAILURE, 0, "%s: currently only 2D datasets supported. "
-          "The input WCS has %d dimensions", __func__, wcs->naxis);
+  if(wcs->naxis!=2) return NAN;
+
+  /* Check if the units of the axis are degrees or not. Currently all FITS
+     images I have worked with use `deg' for degrees. If other alternatives
+     exist, we can add corrections later. */
+  if( strcmp("deg", wcs->cunit[0]) || strcmp("deg", wcs->cunit[1]) )
+    return NAN;
 
   /* Get the pixel scales along each axis in degrees, then multiply. */
-  pixscale=gal_wcs_pixel_scale_deg(wcs);
+  pixscale=gal_wcs_pixel_scale(wcs);
 
   /* Clean up and return the result. */
   out = pixscale[0] * pixscale[1] * 3600.0f * 3600.0f;
@@ -492,62 +613,169 @@ gal_wcs_pixel_area_arcsec2(struct wcsprm *wcs)
 
 
 
+
 /**************************************************************/
 /**********            Array conversion            ************/
 /**************************************************************/
-/* Convert an array of world coordinates to image coordinates. Note that in
-   Gnuastro, each column is treated independently, so the inputs are
-   separate. If `*x==NULL', or `*y==NULL', then space will be allocated for
-   them, otherwise, it is assumed that space has already been
-   allocated. Note that they must each be a 1 dimensional array.
-
-   You can do the conversion in place: just pass the same array as you give
-   to RA and Dec to X and Y. */
-void
-gal_wcs_world_to_img(struct wcsprm *wcs, double *ra, double *dec,
-                     double **x, double **y, size_t size)
+/* Some sanity checks for the WCS conversion functions. */
+static void
+wcs_convert_sanity_check_alloc(gal_data_t *coords, struct wcsprm *wcs,
+                               const char *func, int **stat, double **phi,
+                               double **theta, double **world,
+                               double **pixcrd, double **imgcrd)
 {
-  size_t i;
-  int status, *stat, ncoord=size, nelem=2;
-  double *phi, *theta, *world, *pixcrd, *imgcrd;
+  gal_data_t *tmp;
+  size_t ndim=0, firstsize=0, size=coords->size;
+
+  for(tmp=coords; tmp!=NULL; tmp=tmp->next)
+    {
+      /* Count how many coordinates are given. */
+      ++ndim;
+
+      /* Check the type of the input. */
+      if(tmp->type!=GAL_TYPE_FLOAT64)
+        error(EXIT_FAILURE, 0, "%s: input coordinates must have `float64' "
+              "type", func);
+
+      /* Make sure it has a single dimension. */
+      if(tmp->ndim!=1)
+        error(EXIT_FAILURE, 0, "%s: input coordinates for each dimension "
+              "must each be one dimensional. Coordinate dataset %zu of the "
+              "inputs has %zu dimensions", func, ndim, tmp->ndim);
+
+      /* See if all inputs have the same size. */
+      if(ndim==1) firstsize=tmp->size;
+      else
+        if(firstsize!=tmp->size)
+          error(EXIT_FAILURE, 0, "%s: all input coordinates must have the "
+                "same number of elements. Coordinate dataset %zu has %zu "
+                "elements while the first coordinate has %zu", func, ndim,
+                tmp->size, firstsize);
+    }
+
+  /* See if the number of coordinates given corresponds to the dimensions
+     of the WCS structure. */
+  if(ndim!=wcs->naxis)
+    error(EXIT_FAILURE, 0, "%s: the number of input coordinates (%zu) does "
+          "not match the dimensions of the input WCS structure (%d)", func,
+          ndim, wcs->naxis);
 
   /* Allocate all the necessary arrays. */
-  phi    = gal_data_malloc_array( GAL_TYPE_FLOAT64, size, __func__, "phi");
-  stat   = gal_data_calloc_array( GAL_TYPE_INT32,   size, __func__, "stat");
-  theta  = gal_data_malloc_array( GAL_TYPE_FLOAT64, size, __func__, "theta");
-  world  = gal_data_malloc_array( GAL_TYPE_FLOAT64, 2*size, __func__,
-                                  "world");
-  imgcrd = gal_data_malloc_array( GAL_TYPE_FLOAT64, 2*size, __func__,
-                                  "imgcrd");
-  pixcrd = gal_data_malloc_array( GAL_TYPE_FLOAT64, 2*size, __func__,
-                                  "pixcrd");
+  *phi    = gal_data_malloc_array( GAL_TYPE_FLOAT64, size, __func__, "phi");
+  *stat   = gal_data_calloc_array( GAL_TYPE_INT32,   size, __func__, "stat");
+  *theta  = gal_data_malloc_array( GAL_TYPE_FLOAT64, size, __func__, "theta");
+  *world  = gal_data_malloc_array( GAL_TYPE_FLOAT64, ndim*size, __func__,
+                                   "world");
+  *imgcrd = gal_data_malloc_array( GAL_TYPE_FLOAT64, ndim*size, __func__,
+                                   "imgcrd");
+  *pixcrd = gal_data_malloc_array( GAL_TYPE_FLOAT64, ndim*size, __func__,
+                                   "pixcrd");
+}
 
-  /* Write the values into the allocated contiguous array. */
-  for(i=0;i<size;++i) { world[i*2]=ra[i]; world[i*2+1]=dec[i]; }
 
-  /* Use WCSLIB's `wcss2p'. */
+
+
+
+/* In Gnuastro, each column (coordinate for WCS conversion) is treated as a
+   separate array in a `gal_data_t' that are linked through a linked
+   list. But in WCSLIB, the input is a single array (with multiple
+   columns). This function will convert between the two. */
+static void
+wcs_convert_list_to_array(gal_data_t *list, double *array, int *stat,
+                          size_t ndim, int listtoarray)
+{
+  size_t i, d=0;
+  gal_data_t *tmp;
+
+  for(tmp=list; tmp!=NULL; tmp=tmp->next)
+    {
+      /* Put all this coordinate's values into the single array that is
+         input into or output from WCSLIB. */
+      for(i=0;i<list->size;++i)
+        {
+          if(listtoarray)
+            array[i*ndim+d] = ((double *)(tmp->array))[i];
+          else
+            ((double *)(tmp->array))[i] = stat[i] ? NAN : array[i*ndim+d];
+        }
+
+      /* Increment the dimension. */
+      ++d;
+    }
+}
+
+
+
+
+
+/* Prepare the output of the WCS conversion functions. */
+static gal_data_t *
+wcs_convert_prepare_out(gal_data_t *coords, struct wcsprm *wcs, int inplace)
+{
+  size_t i;
+  gal_data_t *out=NULL;
+  if(inplace)
+    out=coords;
+  else
+    for(i=0;i<wcs->naxis;++i)
+      gal_list_data_add_alloc(&out, NULL, GAL_TYPE_FLOAT64, 1,
+                              &coords->size, NULL, 0, coords->minmapsize,
+                              wcs->ctype[i], wcs->cunit[i], NULL);
+  return out;
+}
+
+
+
+
+
+/* Convert world coordinates to image coordinates given the input WCS
+   structure. The input must be a linked list of data structures of float64
+   (`double') type. The top element of the linked list must be the first
+   coordinate and etc. If `inplace' is non-zero, then the output will be
+   written into the input's allocated space. */
+gal_data_t *
+gal_wcs_world_to_img(gal_data_t *coords, struct wcsprm *wcs, int inplace)
+{
+  gal_data_t *out;
+  int status, *stat=NULL, ncoord=coords->size, nelem=wcs->naxis;
+  double *phi=NULL, *theta=NULL, *world=NULL, *pixcrd=NULL, *imgcrd=NULL;
+
+  /* Some sanity checks. */
+  wcs_convert_sanity_check_alloc(coords, wcs, __func__, &stat, &phi, &theta,
+                                 &world, &pixcrd, &imgcrd);
+
+
+  /* Write the values from the input list of separate columns into a single
+     array (WCSLIB input). */
+  wcs_convert_list_to_array(coords, world, stat, wcs->naxis, 1);
+
+
+  /* Use WCSLIB's wcsp2s for the conversion. */
   status=wcss2p(wcs, ncoord, nelem, world, phi, theta, imgcrd, pixcrd, stat);
   if(status)
     error(EXIT_FAILURE, 0, "%s: wcss2p ERROR %d: %s", __func__, status,
           wcs_errmsg[status]);
 
-  /* For a sanity check:
-  printf("\n\ngal_wcs_world_to_img sanity check:\n");
-  for(i=0;i<size;++i)
-    printf("world (%f, %f) --> pix (%f, %f), [stat: %d]\n",
-           world[i*2], world[i*2+1], pixcrd[i*2], pixcrd[i*2+1], stat[i]);
+
+  /* For a sanity check.
+  {
+    size_t i;
+    printf("\n\n%s sanity check:\n", __func__);
+    for(i=0;i<coords->size;++i)
+      printf("(%g, %g) --> (%g, %g), [stat: %d]\n", world[i*2], world[i*2+1],
+             pixcrd[i*2], pixcrd[i*2+1], stat[i]);
+  }
   */
 
-  /* Allocate the output arrays if they were not already allocated. */
-  if(*x==NULL) *x=gal_data_malloc_array(GAL_TYPE_FLOAT64, size, __func__,"x");
-  if(*y==NULL) *y=gal_data_malloc_array(GAL_TYPE_FLOAT64, size, __func__,"y");
 
-  /* Put the values into the output arrays. */
-  for(i=0;i<size;++i)
-    {
-      (*x)[i] = stat[i] ? NAN : pixcrd[i*2];
-      (*y)[i] = stat[i] ? NAN : pixcrd[i*2+1];
-    }
+  /* Allocate the output arrays if they were not already allocated. */
+  out=wcs_convert_prepare_out(coords, wcs, inplace);
+
+
+  /* Write the output from a single array (WCSLIB output) into the output
+     list of this function. */
+  wcs_convert_list_to_array(out, pixcrd, stat, wcs->naxis, 0);
+
 
   /* Clean up. */
   free(phi);
@@ -555,35 +783,32 @@ gal_wcs_world_to_img(struct wcsprm *wcs, double *ra, double *dec,
   free(theta);
   free(world);
   free(pixcrd);
+
+  /* Return the output list of coordinates. */
+  return out;
 }
 
 
 
 
 
-/* Similar to `gal_wcs_world_to_img' but converts image coordinates into
-   world coordinates. */
-void
-gal_wcs_img_to_world(struct wcsprm *wcs, double *x, double *y,
-                     double **ra, double **dec, size_t size)
+/* Similar to `gal_wcs_world_to_img'. */
+gal_data_t *
+gal_wcs_img_to_world(gal_data_t *coords, struct wcsprm *wcs, int inplace)
 {
-  size_t i;
-  int status, *stat, ncoord=size, nelem=2;
-  double *phi, *theta, *world, *pixcrd, *imgcrd;
+  gal_data_t *out;
+  int status, *stat=NULL, ncoord=coords->size, nelem=wcs->naxis;
+  double *phi=NULL, *theta=NULL, *world=NULL, *pixcrd=NULL, *imgcrd=NULL;
 
-  /* Allocate all the necessary arrays. */
-  phi    = gal_data_malloc_array( GAL_TYPE_FLOAT64, size, __func__, "phi");
-  stat   = gal_data_calloc_array( GAL_TYPE_INT32,   size, __func__, "stat");
-  theta  = gal_data_malloc_array( GAL_TYPE_FLOAT64, size, __func__, "theta");
-  world  = gal_data_malloc_array( GAL_TYPE_FLOAT64, 2*size, __func__,
-                                  "world");
-  imgcrd = gal_data_malloc_array( GAL_TYPE_FLOAT64, 2*size, __func__,
-                                  "imgcrd");
-  pixcrd = gal_data_malloc_array( GAL_TYPE_FLOAT64, 2*size, __func__,
-                                  "pixcrd");
+  /* Some sanity checks. */
+  wcs_convert_sanity_check_alloc(coords, wcs, __func__, &stat, &phi, &theta,
+                                 &world, &pixcrd, &imgcrd);
+
+
+  /* Write the values from the input list of separate columns into a single
+     array (WCSLIB input). */
+  wcs_convert_list_to_array(coords, pixcrd, stat, wcs->naxis, 1);
 
-  /* Write the values into the allocated contiguous array. */
-  for(i=0;i<size;++i) { pixcrd[i*2]=x[i]; pixcrd[i*2+1]=y[i]; }
 
   /* Use WCSLIB's wcsp2s for the conversion. */
   status=wcsp2s(wcs, ncoord, nelem, pixcrd, imgcrd, phi, theta, world, stat);
@@ -591,25 +816,26 @@ gal_wcs_img_to_world(struct wcsprm *wcs, double *x, double *y,
     error(EXIT_FAILURE, 0, "%s: wcsp2s ERROR %d: %s", __func__, status,
           wcs_errmsg[status]);
 
-  /* For a sanity check:
-  printf("\n\ngal_wcs_img_to_world sanity check:\n");
-  for(i=0;i<size;++i)
-    printf("img (%f, %f) --> world (%f, %f), [stat: %d]\n",
-           pixcrd[i*2], pixcrd[i*2+1], world[i*2], world[i*2+1], stat[i]);
+
+  /* For a sanity check.
+  {
+    size_t i;
+    printf("\n\n%s sanity check:\n", __func__);
+    for(i=0;i<coords->size;++i)
+      printf("(%g, %g) --> (%g, %g), [stat: %d]\n", pixcrd[i*2], pixcrd[i*2+1],
+             world[i*2],  world[i*2+1], stat[i]);
+  }
   */
 
-  /* Allocate the output arrays if they were not already allocated. */
-  if(*ra==NULL)
-    *ra  = gal_data_malloc_array(GAL_TYPE_FLOAT64, size, __func__, "ra");
-  if(*dec==NULL)
-    *dec = gal_data_malloc_array(GAL_TYPE_FLOAT64, size, __func__, "dec");
 
-  /* Put the values into the output arrays. */
-  for(i=0;i<size;++i)
-    {
-      (*ra)[i]  = stat[i] ? NAN : world[i*2];
-      (*dec)[i] = stat[i] ? NAN : world[i*2+1];
-    }
+  /* Allocate the output arrays if they were not already allocated. */
+  out=wcs_convert_prepare_out(coords, wcs, inplace);
+
+
+  /* Write the output from a single array (WCSLIB output) into the output
+     list of this function. */
+  wcs_convert_list_to_array(out, world, stat, wcs->naxis, 0);
+
 
   /* Clean up. */
   free(phi);
@@ -617,4 +843,8 @@ gal_wcs_img_to_world(struct wcsprm *wcs, double *x, double *y,
   free(theta);
   free(world);
   free(pixcrd);
+
+
+  /* Return the output list of coordinates. */
+  return out;
 }
