@@ -19,7 +19,7 @@
 # Preliminaries
 # =============
 #
-# Set the variabels (The executable is in the build tree). Do the
+# Set the variables (The executable is in the build tree). Do the
 # basic checks to see if the executable is made or if the defaults
 # file exists (basicchecks.sh is in the source tree).
 psf=psf.fits
@@ -41,12 +41,13 @@ execname=../bin/$prog/ast$prog
 #
 #   - The input data was not made (for example the test that created the
 #     data file failed).
-if [ ! -f $execname ] || [ ! -f $img ] || [ ! -f $psf ]; then exit 77; fi
-
+if [ ! -f $execname ]; then echo "$execname not created."; exit 77; fi
+if [ ! -f $img      ]; then echo "$img does not exist.";   exit 77; fi
+if [ ! -f $psf      ]; then echo "$psf does not exist.";   exit 77; fi
 
 
 
 
 # Actual test script
 # ==================
-$execname $img --kernel=$psf --spatial --output=convolve_spatial.fits
+$execname $img --kernel=$psf --domain=spatial --output=convolve_spatial.fits
