@@ -52,4 +52,4 @@ if [ ! -f $cat      ]; then echo "$cat does not exist.";   exit 77; fi
 
 # Actual test script
 # ==================
-$execname $cat --naxis1=100 --naxis2=100 --crpix2=-99
+$execname $cat --naxis=100,100 --crpix=1,-99
